package mqtrcfrmt.parser;

import java.io.*;

public class MessageBufferUtil {
	
	public static void messageBufferAddByte(TrcCtl trcCtl) {
		MessageBuffer mb;
		
		// Allocate a MessageBuffer for trcCtl.messageBufferList, if empty
		if (trcCtl.messageBufferList.size() == 0)
		{
			mb = new MessageBuffer();
			trcCtl.messageBufferList.addLast(mb);
		}
		
		mb = trcCtl.messageBufferList.getLast();
		
		// If we have filled our current MessageBuffer, create a new one and add it to the end of our list
		if (mb.messageBufferOffset >= mb.messageBufferMax)
		{
			mb = new MessageBuffer();
			trcCtl.messageBufferList.addLast(mb);
		}
		
		mb = trcCtl.messageBufferList.getLast();
		
		if (trcCtl.dsBufferOffset > 0)
		{
			mb.messageBuffer[mb.messageBufferOffset] = trcCtl.dsBuffer[trcCtl.dsBufferOffset-1];
			mb.messageBufferOffset++;
		}

	}
	
	public static void msg2File(TrcCtl trcCtl) {
		try {
			String outputFile = null;
			
			if (trcCtl.msg2FileGlobal)
			{
				trcCtl.msg2FileGlobalCt++;
				outputFile = "msg2File" + "_" + trcCtl.pidTid + "_" + trcCtl.msg2FileGlobalStr + "_" + trcCtl.msg2FileGlobalCt;
			}
			else
			{
				outputFile = trcCtl.msg2FileStr;
			}
			
			OutputStream outputStream = new FileOutputStream(outputFile);
			
			for (MessageBuffer mb:trcCtl.messageBufferList)
			{
				for (int i=0; i < mb.messageBufferOffset; i++)
					outputStream.write(mb.messageBuffer[i]);
			}
			
			outputStream.close();
			
		} catch (Exception ex) {
			System.err.println("Error encountered when writing to msg2File " + trcCtl.msg2FileStr);
			System.err.println("Error: " + ex);
        	ex.printStackTrace();
            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
              System.err.println("... Caused by ");
              t.printStackTrace();
            }
            System.err.flush();
            System.exit(-1);
		}
	}
	
	public static void messageSearch(TrcCtl trcCtl) {
		
		// We will assume 10 times the trcCtl.messageSearchStrMax is enough to hold the bytes of String trcCtl.messageSearchStrMax
		byte [] srchBytes1 = new byte[trcCtl.messageSearchStrMax * 10];
		byte [] srchBytes2 = new byte[trcCtl.messageSearchStrMax * 10];
		byte [] srchBytes3 = new byte[trcCtl.messageSearchStrMax * 10];
		
		boolean str1Match = false;
		boolean str2Match = false;
		boolean str3Match = false;
		
		int len1 = 0;
		int len2 = 0;
		int len3 = 0;
		
    	// Convert our search strings to a byte array
		int strCt = 0;
		String str1 = null;
		
		if (trcCtl.messageSearchStrCt == 1)
    		strCt = 1;
    	
    	if (trcCtl.messageSearchStrCt == 3)
    		strCt = 2;
    	
       	if (trcCtl.messageSearchStrCt == 4)
    		strCt = 3;
    	
		for (int x = 1; x <= strCt; x++)
		{
			if (x == 1)
				str1 = trcCtl.messageSearchStr;
			
			if (x == 2)
				str1 = trcCtl.messageSearchStr2;
			
			if (x == 3)
				str1 = trcCtl.messageSearchStr3;

			int len = str1.length();
			
			// if str1 is > 3, an even length, and starts with 0x, treat this as a string of hex bytes to search for
			if (len > 3 && len % 2 == 0 && str1.substring(0, 1).equals("0") && str1.substring(1, 2).equals("x"))
			{
				int j = 0;
				for (int i = 2; i < len; i += 2) {
                    if (x == 1)
                    	srchBytes1[j] = (byte) ((Character.digit(str1.charAt(i), 16) << 4)
		        			+ Character.digit(str1.charAt(i+1), 16));
                    if (x == 2)
                    	srchBytes2[j] = (byte) ((Character.digit(str1.charAt(i), 16) << 4)
		        			+ Character.digit(str1.charAt(i+1), 16));
                    if (x == 3)
                    	srchBytes3[j] = (byte) ((Character.digit(str1.charAt(i), 16) << 4)
		        			+ Character.digit(str1.charAt(i+1), 16));
					j++;
				}
				len = (len - 2) / 2;
			}
			else
			{
				// convert str1 to bytes using trcCtl.inputFileCharset
				try {
					if (x == 1)
						srchBytes1 = str1.getBytes(trcCtl.inputFileCharset);
					if (x == 2)
						srchBytes2 = str1.getBytes(trcCtl.inputFileCharset);
					if (x == 3)
						srchBytes3 = str1.getBytes(trcCtl.inputFileCharset);
				} catch (Exception ex) {
					System.err.println("Error encountered when converting to bytes for String messageSearch " + str1 + " using charset " + trcCtl.inputFileCharset);
		        	ex.printStackTrace();
		            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
		              System.err.println("... Caused by ");
		              t.printStackTrace();
		            }
		            System.err.flush();
		            System.exit(-1);
				}
			}
			
			if (x == 1)
				len1 = len;
			
			if (x == 2)
				len2 = len;
			
			if (x == 3)
				len3 = len;
					
		}
		
        // Search our message buffer for hits for the 1-3 search strings
		int hitCt1 = 0;
		int hitCt2 = 0;
		int hitCt3 = 0;
		int offset = 0;
		
		for (MessageBuffer mb:trcCtl.messageBufferList)
		{
			for (int i=0; i < mb.messageBufferOffset; i++)
			{
				offset++;
				if (trcCtl.messageSearchStrCt > 0)
				{
					if (srchBytes1[hitCt1] == mb.messageBuffer[i])
						hitCt1++;
					else
						hitCt1 = 0;
					if (hitCt1 == len1)
					{
						TrcParser.writeOutputFile(trcCtl, "msgSearch: string1 hit at message offset " + Integer.toHexString(offset - len1));
						hitCt1 = 0;
						str1Match = true;
					}
				}
				if (trcCtl.messageSearchStrCt > 2)
				{
					if (srchBytes2[hitCt2] == mb.messageBuffer[i])
						hitCt2++;
					else
						hitCt2 = 0;
					if (hitCt2 == len2)
					{
						TrcParser.writeOutputFile(trcCtl, "msgSearch: string2 hit at message offset " + Integer.toHexString(offset - len2));
						hitCt2 = 0;
						str2Match = true;
					}
				}
				if (trcCtl.messageSearchStrCt > 3)
				{
					if (srchBytes3[hitCt3] == mb.messageBuffer[i])
						hitCt3++;
					else
						hitCt3 = 0;
					if (hitCt3 == len3)
					{
						TrcParser.writeOutputFile(trcCtl, "msgSearch: string3 hit at message offset " + Integer.toHexString(offset - len3));
						hitCt3 = 0;
						str3Match = true;
					}
				}
			}
		}
		
        // Report if a multi-string search was successful
		if (trcCtl.messageSearchStrCt == 3)
		{
			if (trcCtl.messageSearchStr3.equals("AND") && (str1Match && str2Match))
				TrcParser.writeOutputFile(trcCtl, "msgSearch: multi-string match");
			
			if (trcCtl.messageSearchStr3.equals("OR") && (str1Match || str2Match))
				TrcParser.writeOutputFile(trcCtl, "msgSearch: multi-string match");
		}
		
		if (trcCtl.messageSearchStrCt == 4)
		{
			if (trcCtl.messageSearchStr4.equals("AND") && (str1Match && str2Match  && str3Match))
				TrcParser.writeOutputFile(trcCtl, "msgSearch: multi-string match");
			
			if (trcCtl.messageSearchStr4.equals("OR") && (str1Match || str2Match || str3Match))
				TrcParser.writeOutputFile(trcCtl, "msgSearch: multi-string match");			
		}
			
	}

}
