package mqtrcfrmt.parser;

public class MessageParsingUtil {
	
	public static void messageParsing(TrcCtl trcCtl, int CCSID, String endianness) {
		switch (CCSID) {
        case 819:
        	iso88591Check(trcCtl);
        	break;
        case 1200:
        	utf16Check(trcCtl, endianness);
        	break;
        case 1208:
        	utf8Check(trcCtl);
            break;
        default:
        	break;
        }
	}
	
	public static void iso88591Check(TrcCtl trcCtl) {
		if (trcCtl.dsBufferOffset > 0)
		{
			byte b = trcCtl.dsBuffer[trcCtl.dsBufferOffset-1];
			
			int x = b & 0xff;
			
			//Check if byte is ASCII
			if ((x & 0x80) == 0)
				trcCtl.messageParsingASCIIBytes++;
			
			//Check if byte is in an invalid range for ISO-8859-1
			if ((x >= 0 && x <= 31) || (x >= 127 && x <= 160))
			{
				trcCtl.messageParsingInvalidBytes++;
				MessageParsingUtil.messageParsingTrackerAddItem(trcCtl, "invalid", trcCtl.messageParsingTotalBytes);
			}
			
			trcCtl.messageParsingTotalBytes++;
		}
	}
	
	public static void utf16Check(TrcCtl trcCtl, String endianness) {
		if (trcCtl.dsBufferOffset > 0)
		{
			boolean evenCd = false;
			
			byte b = trcCtl.dsBuffer[trcCtl.dsBufferOffset-1];
			
			int x = b & 0xff;
			
			if ((trcCtl.messageParsingTotalBytes + 1) % 2 == 0)
				evenCd = true;
			
			// Check for BOM in first two bytes of message
			if (trcCtl.messageParsingTotalBytes == 0)
			{
				if (x == 254)
					trcCtl.messageParsingBomBECt++;
				
				if (x == 255)
					trcCtl.messageParsingBomLECt++;
			}
			
			if (trcCtl.messageParsingTotalBytes == 1)
			{
				if (x == 254)
					trcCtl.messageParsingBomLECt++;
				
				if (x == 255)
					trcCtl.messageParsingBomBECt++;
			}
			
			// BOM in message will override input endianness
			if (trcCtl.messageParsingBomBECt == 2)
				endianness = "big";
			
			if (trcCtl.messageParsingBomLECt == 2)
				endianness = "little";
			
			// Validate if we previously detected a surrogate pair
			if (trcCtl.messageParsingSpFlag)
			{
				trcCtl.messageParsingSpContCt++;
				
				if (trcCtl.messageParsingSpContCt == 2)
				{
					// Check for start of low surrogate (DC - DF)
				    if ( ( (x & 0x80) != 0 && (x & 0x40) != 0 && (x & 0x20) == 0 && (x & 0x10) != 0) &&
				         ( ( (x & 0x08) != 0 &&  (x & 0x04) != 0 && (x & 0x02) == 0 && (x & 0x01) == 0) ||
				             ( (x & 0x08) != 0 &&  (x & 0x04) != 0 && (x & 0x02) == 0 &&  (x & 0x01) != 0 ) ||
				             ( (x & 0x08) != 0 &&  (x & 0x04) != 0 &&  (x & 0x02) != 0 && (x & 0x01) == 0) ||
				             ( (x & 0x08) != 0 &&  (x & 0x04) != 0 &&  (x & 0x02) != 0 &&  (x & 0x01) != 0) ) )
				    {
				    	// we are at the end of the surrogate pair at this point, if little endian
				    	if (endianness.equals("little"))
				        {
				    		trcCtl.messageParsingSpFlag=false;
				    		trcCtl.messageParsingSpTotCt++;
				        }
				    }
				    else
				    {
				    	trcCtl.messageParsingSpFlag=false;
				    	trcCtl.messageParsingInvalidBytes++;
				    	MessageParsingUtil.messageParsingTrackerAddItem(trcCtl, "invalid", trcCtl.messageParsingTotalBytes);
				    }
				}
				
				if (trcCtl.messageParsingSpContCt == 3)
				{
					trcCtl.messageParsingSpFlag=false;
					trcCtl.messageParsingSpTotCt++;
				}
				
			}
						
			// Check for start of high surrogate (D8 - DB), if we are not already parsing a surrogate pair
			if ( (!trcCtl.messageParsingSpFlag) && 
				       ((endianness.equals("big") && !evenCd) || (endianness.equals("little") && evenCd)) && 
				       ( (x & 0x80) != 0 && (x & 0x40) != 0 && (x & 0x20) == 0 && (x & 0x10) != 0) && 
				       ( ( (x & 0x08) != 0 && (x & 0x04) == 0 && (x & 0x02) == 0 && (x & 0x01) == 0) ||
				         ( (x & 0x08) != 0 && (x & 0x04) == 0 && (x & 0x02) == 0 &&  (x & 0x01) != 0) ||
				         ( (x & 0x08) != 0 && (x & 0x04) == 0 &&  (x & 0x02) != 0 && (x & 0x01) == 0) ||
				         ( (x & 0x08) != 0 && (x & 0x04) == 0 &&  (x & 0x02) != 0 &&  (x & 0x01) != 0) ) ) 
			{
				trcCtl.messageParsingSpFlag=true;
		    	trcCtl.messageParsingSpContCt=0;
		    	MessageParsingUtil.messageParsingTrackerAddItem(trcCtl, "surrogatePair", trcCtl.messageParsingTotalBytes);
			}
			
			// Check for ASCII Byte
			if (((x & 0x80) == 0 && !trcCtl.messageParsingSpFlag) && 
				((endianness.equals("big") && evenCd) || (endianness.equals("little") && !evenCd)))
			{
				if (endianness.equals("little"))
				{
					trcCtl.messageParsingASCIILeadByte = trcCtl.messageParsingTotalBytes;
				}
				else
				{
					if (trcCtl.messageParsingASCIILeadByte >= 0 && trcCtl.messageParsingTotalBytes == trcCtl.messageParsingASCIILeadByte + 1)
						trcCtl.messageParsingASCIIBytes++;
				}
			}
			
			if ((x == 0 && !trcCtl.messageParsingSpFlag) && 
				((endianness.equals("big") && !evenCd) || (endianness.equals("little") && evenCd)))
			{
				if (endianness.equals("little"))
				{
					if (trcCtl.messageParsingASCIILeadByte >= 0 && trcCtl.messageParsingTotalBytes == trcCtl.messageParsingASCIILeadByte + 1)
						trcCtl.messageParsingASCIIBytes++;
				}
				else
				{
					trcCtl.messageParsingASCIILeadByte = trcCtl.messageParsingTotalBytes;
				}
			}
			
			trcCtl.messageParsingTotalBytes++;
		}
	}
	
	public static void utf8Check(TrcCtl trcCtl) {
		if (trcCtl.dsBufferOffset > 0)
		{
			boolean validCd = false;
			boolean mbErrorCd = false;
			
			byte b = trcCtl.dsBuffer[trcCtl.dsBufferOffset-1];
			
			int x = b & 0xff;
			
			// if MBLength previously set, process byte as a continuation byte
			if (trcCtl.messageParsingMBLength != 0)
			{
			    // Check for high bits of 10 for continuation multi byte 
				if ( (x & 0x80) != 0 && (x & 0x40) == 0)
			    {
					validCd=true;
			        trcCtl.messageParsingMBContCt--;
			        if (trcCtl.messageParsingMBContCt == 0)
			        {
			           switch (trcCtl.messageParsingMBLength)
			           {
			           case 2:
			        	   trcCtl.messageParsingMB2TotalBytes++;
			        	   break;
			           case 3:
			        	   trcCtl.messageParsingMB3TotalBytes++;
			        	   break;
			           case 4:
			        	   trcCtl.messageParsingMB4TotalBytes++;
			        	   break;
			           default:
			        	   break;
			           }
			           trcCtl.messageParsingMBLength=0;
			        }
			    }
			    else
			    {
			    	trcCtl.messageParsingMBLength=0;
			    	trcCtl.messageParsingMBContCt=0;
			    	mbErrorCd=true;
			    }
				
			}
			
			// Check for high bits of 110 for leading 2 multi byte
			if ( (x & 0x80) != 0 && (x & 0x40) != 0 && (x & 0x20) == 0)
			{

				// Check for invalid bytes of C0 and C1
			    if ( (x & 0x10) == 0 && (x & 0x08) == 0 && (x & 0x04) == 0 && (x & 0x02) == 0)
			        ;
			    else
			    {
			    	trcCtl.messageParsingMBLength=2;
			    	trcCtl.messageParsingMBContCt=1;
			        validCd=true;
			        MessageParsingUtil.messageParsingTrackerAddItem(trcCtl, "multiByte2", trcCtl.messageParsingTotalBytes);
			    }
			}
			
			// Check for high bits of 1110 for leading 3 multi byte
			if ( (x & 0x80) != 0 && (x & 0x40) != 0 && (x & 0x20) != 0 && (x & 0x10) == 0)
			{
				 trcCtl.messageParsingMBLength=3;
				 trcCtl.messageParsingMBContCt=2;
			     validCd=true;
			     MessageParsingUtil.messageParsingTrackerAddItem(trcCtl, "multiByte3", trcCtl.messageParsingTotalBytes);
			}
			 
			// Check for high bits of 11110 for leading 4 multi byte
			if ( (x & 0x80) != 0 && (x & 0x40) != 0 && (x & 0x20) != 0 && (x & 0x10) != 0 && (x & 0x08) == 0)
			{
			     // Check for invalid bytes of F5 - FF
			     if ( (x & 0x04) != 0 && ((x & 0x02) != 0 || (x & 0x01) != 0) )
			     {
			    	 
			     }
			     else
			     {
			    	 trcCtl.messageParsingMBLength=4;
					 trcCtl.messageParsingMBContCt=3;
				     validCd=true;
				     MessageParsingUtil.messageParsingTrackerAddItem(trcCtl, "multiByte4", trcCtl.messageParsingTotalBytes);
			     }
			}
			
			// Check if high bit is 0, then ascii
			if ((x & 0x80) == 0)
			{
				trcCtl.messageParsingASCIIBytes++;
				validCd=true;
			}
			
			if (!validCd || mbErrorCd)
			{
				trcCtl.messageParsingInvalidBytes++;
				MessageParsingUtil.messageParsingTrackerAddItem(trcCtl, "invalid", trcCtl.messageParsingTotalBytes);
			}
			
			trcCtl.messageParsingTotalBytes++;
		
		}
		
	}
	
	public static void messageParsingTrackerAddItem(TrcCtl trcCtl, String type, int offset) {

		MessageParsingTracker mpt = new MessageParsingTracker();
		mpt.type = type;
		mpt.offset = offset;
		trcCtl.messageParsingTrackerList.addLast(mpt);

	}
	
	public static void printMessageParsing(TrcCtl trcCtl) {
		if (TrcParser.messageParsingValue == 819)
		{
			TrcParser.writeOutputFile(trcCtl, "msg-parser ISO-8859-1 Totals: Pid:" + trcCtl.pidTid + " Format:" + trcCtl.format + " CCSID:" + trcCtl.CCSID + " API:" + trcCtl.mqAPI + " " + trcCtl.mqAPIType + " Byte:" + trcCtl.messageParsingTotalBytes + " ASCII:" + trcCtl.messageParsingASCIIBytes + " Inv:" + trcCtl.messageParsingInvalidBytes);
			MessageParsingUtil.printMessagingParsingTracker(trcCtl);
		}
		
		if (TrcParser.messageParsingValue == 1200)
		{
			TrcParser.writeOutputFile(trcCtl, "msg-parser UTF-16 Totals: Pid:" + trcCtl.pidTid + " Format:" + trcCtl.format + " CCSID:" + trcCtl.CCSID + " API:" + trcCtl.mqAPI + " " + trcCtl.mqAPIType + " Byte:" + trcCtl.messageParsingTotalBytes + " ASCII:" + trcCtl.messageParsingASCIIBytes + " SP:" + trcCtl.messageParsingSpTotCt + " Inv:" + trcCtl.messageParsingInvalidBytes);
			MessageParsingUtil.printMessagingParsingTracker(trcCtl);
		}
		
		if (TrcParser.messageParsingValue == 1208)
		{
			TrcParser.writeOutputFile(trcCtl, "msg-parser UTF-8 Totals: Pid:" + trcCtl.pidTid + " Format:" + trcCtl.format + " CCSID:" + trcCtl.CCSID + " API:" + trcCtl.mqAPI + " " + trcCtl.mqAPIType + " Byte:" + trcCtl.messageParsingTotalBytes + " ASCII:" + trcCtl.messageParsingASCIIBytes + " MB2:" + trcCtl.messageParsingMB2TotalBytes + " MB3:" + trcCtl.messageParsingMB3TotalBytes + " MB4:" + trcCtl.messageParsingMB4TotalBytes + " Inv:" + trcCtl.messageParsingInvalidBytes);
			MessageParsingUtil.printMessagingParsingTracker(trcCtl);
		}
			
			
	}
	
	public static void printMessagingParsingTracker(TrcCtl trcCtl) {

		if (trcCtl.messageParsingTrackerList.size() == 0)
			return;
		
		int i=0;
		boolean needToPrint=true;
		StringBuilder builder = new StringBuilder();
		builder.append("msg-parser Byte Analysis: ");
		
		for (MessageParsingTracker mpt:trcCtl.messageParsingTrackerList)
		{
			needToPrint=true;
			
			switch (mpt.type) {
			case "invalid" :
				// str1 = str1 + Integer.toHexString(mpt.offset) + "-INV,";
				builder.append(Integer.toHexString(mpt.offset));
				builder.append("-INV,");
				break;
			case "multiByte2" :
				// str1 = str1 + Integer.toHexString(mpt.offset) + "-MB2,";
				builder.append(Integer.toHexString(mpt.offset));
				builder.append("-MB2,");
				break;
			case "multiByte3" :
				// str1 = str1 + Integer.toHexString(mpt.offset) + "-MB3,";
				builder.append(Integer.toHexString(mpt.offset));
				builder.append("-MB3,");
				break;
			case "multiByte4" :
				// str1 = str1 + Integer.toHexString(mpt.offset) + "-MB4,";
				builder.append(Integer.toHexString(mpt.offset));
				builder.append("-MB4,");
				break;
			case "surrogatePair" :
				// str1 = str1 + Integer.toHexString(mpt.offset) + "-SP,";
				builder.append(Integer.toHexString(mpt.offset));
				builder.append("-SP,");
				break;
			default :
				break;
			}
			if (i == 100)
			{
				i=0;
				needToPrint=false;
				// TrcParser.writeOutputFile(trcCtl, str1);
				TrcParser.writeOutputFile(trcCtl, builder.toString());
				builder = new StringBuilder();
				builder.append(trcCtl.pidTid);
				builder.append(" msg-parser: Byte Analysis - ");
				// TrcParser.writeOutputFile(trcCtl, trcCtl.pidTid + " msg-parser: Byte Analysis - ");
			}
			i++;			
		}
		if (needToPrint)
			TrcParser.writeOutputFile(trcCtl, builder.toString());
		
	}
}
