package mqtrcfrmt.parser;

import java.util.*;
import java.io.*;
import java.nio.ByteOrder;
import java.nio.ByteBuffer;

public class TrcParser {

	static String inputFile;
	static String outputFileSuffix;
	static String inputFileCharset;
	static boolean fileCharSetGiven;
	static String endianness;
	static boolean endiannessGiven;
	static String traceType;
	static boolean traceTypeGiven;
    static String printCharStr;
    static boolean printCharGiven;
	static String pointerLengthStr;
	static boolean pointerLengthGiven;
	static short pointerLength;
	static String messageParsingStr;
	static boolean messageParsingGiven;
	static int messageParsingValue;
	static boolean messageSearchGiven;
	static String messageSearchStr;
	static String messageSearchStr2;
	static String messageSearchStr3;
	static String messageSearchStr4;
	static short messageSearchStrCt;
	static int messageSearchStrMax;
	static boolean msg2FileGlobalGiven;
	static String msg2FileGlobalStr;
	static String dumpLinePrefix;
	static boolean dumpLineWidthGiven;
	static String dumpLineWidthStr;
	static short dumpLineWidth;
	static String constField;
	static boolean constFieldGiven;
	static String constValueStr;
	static boolean constValueGiven;
	static int constValue;
	static String reportStr;
	static boolean reportGiven;
	static long statusInterval;
	static volatile long bytesProcessed;
	static long totalBytes;
	static volatile long lastReportedTimeMillis;
	final static char[] hexArray = "0123456789abcdef".toCharArray();
	final static char[] hexArrayWin = "0123456789ABCDEF".toCharArray();
			
	public static void main(String[] args) {
		// default optional input parameters
		TrcParser.outputFileSuffix = ".2";
		TrcParser.inputFileCharset = "UTF-8";
		TrcParser.endianness = "little";
		TrcParser.traceType = "undefined";
		TrcParser.printCharStr = "ASCII";
		TrcParser.pointerLength = 8;
		TrcParser.messageParsingValue = 0;
		TrcParser.messageSearchStr = null;
		TrcParser.messageSearchStr2 = null;
		TrcParser.messageSearchStr3 = null;
		TrcParser.messageSearchStr4 = null;
		TrcParser.messageSearchStrCt = 0;
		TrcParser.messageSearchStrMax = 1000;
		TrcParser.dumpLinePrefix = "0x";
		TrcParser.dumpLineWidth = 16;
		TrcParser.reportStr = "none";
		TrcParser.statusInterval = 0;
		TrcParser.bytesProcessed = 0;
		TrcParser.totalBytes = 0;
		
		TrcParser trcParser = new TrcParser();
		
		System.out.println("mqtrcfrmt version 1.0.10");
		
		parseArgs(args);
		
		// constant field and value, if provided
		if (TrcParser.constFieldGiven || TrcParser.constValueGiven)
		{
			if (TrcParser.constFieldGiven && TrcParser.constField.equals("?"))
			{	
				TrcParser.listConstFieldValidValues();
				System.exit(1);
			}
			
			if (!TrcParser.constFieldGiven || !TrcParser.constValueGiven)
			{
				System.err.println("constantField (-f) and constantValue (-v) must be specified together!");
				System.err.flush();
				System.exit(-1);
			}
			
			TrcParser.constValue = 0;
			
			if (TrcParser.checkDigit(TrcParser.constValueStr))
				TrcParser.constValue = Integer.parseInt(TrcParser.constValueStr);
			
			System.out.println("Field " + TrcParser.constField + " for value " + TrcParser.constValueStr + " resolves to:");
			TrcParser.writeConstants(null, "   ", TrcParser.constField, TrcParser.constValue, TrcParser.constValueStr);
			
			System.exit(1);
			
		}
		
        if (TrcParser.inputFile == null)
        {
			System.err.println("inputFile (-i) is a required input for this invocation of mqtrcfrmt!");
			printUsage();
			System.err.flush();
			System.exit(-1);
        }
		
		// endianness setting
		if (TrcParser.endiannessGiven)
		{
			if (TrcParser.endianness.equals("little") || TrcParser.endianness.equals("big"))
				System.out.println("endianness (-e) has been explicitly set to " + TrcParser.endianness + ".");
			else
			{	
				System.out.println("endianness (-e) explicit setting of " + TrcParser.endianness + " is invalid.  Defaulting to little.");
				TrcParser.endianness = "little";
			}
		}
		else
		{
		     if (ByteOrder.nativeOrder().equals(ByteOrder.BIG_ENDIAN)) 
		     {
		    	 TrcParser.endianness = "big";
		    	 System.out.println("Endianness was not explicitly set. Defaulting to big endian.");
		     }
		     else 
		     {
		    	 TrcParser.endianness = "little";
		    	 System.out.println("Endianness was not explicitly set. Defaulting to little endian.");
		     }
		}
		
		// traceType setting
		if (TrcParser.traceTypeGiven)
		{
			if (TrcParser.traceType.equals("unix") || TrcParser.traceType.equals("windows") || TrcParser.traceType.equals("aat"))
			{
				System.out.println("traceType (-t) has been explicitly set to " + TrcParser.traceType + ".");
				if (TrcParser.traceType.equals("aat"))
				{
					System.out.println("dumpLinePrefix is hard coded to 0000 for aat");
					TrcParser.dumpLinePrefix = "0000";
				}
			}
			else
			{
				System.out.println("traceType (-t) explicit setting is not unix, windows, or aat.  Defaulting to undefined.");
				TrcParser.traceType = "undefined";
			}
		}
		else
		{
			System.out.println("traceType (-t) was not specified.  Defaulting to undefined.");
			TrcParser.traceType = "undefined";
		}
		
		// printChar setting
		if (TrcParser.printCharGiven)
		{
			if (TrcParser.printCharStr.equals("ASCII") || TrcParser.printCharStr.equals("Linux_x86") || TrcParser.printCharStr.equals("none"))
				System.out.println("printChar (-a) has been explicitly set to " + TrcParser.printCharStr + ".");
			else
			{
				System.out.println("printChar (-a) value is not supported.  Defaulting to ASCII.");
				TrcParser.printCharStr = "ASCII";
			}
		}
		
		// inputFileCharSet setting
		if (TrcParser.fileCharSetGiven)
			System.out.println("inputFileCharSet (-c) has been explicitly set to " + TrcParser.inputFileCharset + ".");
		else
		{
			if (TrcParser.traceType.equals("windows"))
			{
				System.out.println("inputFileCharSet (-c) not explicitly set. Defaulting to Windows-1252.");
				TrcParser.inputFileCharset="Windows-1252";
			}
			else
			{
				System.out.println("inputFileCharSet (-c) not explicitly set. Defaulting to UTF-8.");
				TrcParser.inputFileCharset="UTF-8";
			}
		}
		
		// pointerLength setting
		if (TrcParser.pointerLengthGiven)
		{
			if (TrcParser.checkDigit(TrcParser.pointerLengthStr))
			{
				TrcParser.pointerLength = Short.parseShort(TrcParser.pointerLengthStr);
				if (TrcParser.pointerLength != 4 && TrcParser.pointerLength != 8)
				{
					System.err.println("pointerLength (-p) must be either 4 or 8!");
					System.err.flush();
					System.exit(-1);
				}
				System.out.println("pointerLength (-p) has been explicitly set to " + TrcParser.pointerLength + ".");
			}
			else
			{
				System.err.println("pointerLength (-p) must be a number!");
				System.err.flush();
				System.exit(-1);
			}
		}
		else
		{
			System.out.println("pointerLength (-p) not explicitly set. Defaulting to 8.");
			TrcParser.pointerLength=8;
		}
		
		// dumpLineWidth setting
		if (TrcParser.dumpLineWidthGiven)
		{
			if (TrcParser.checkDigit(TrcParser.dumpLineWidthStr))
				TrcParser.dumpLineWidth = Short.parseShort(TrcParser.dumpLineWidthStr);
			else
			{
				System.err.println("dumpLineWidth (-w) must be a number!");
				System.err.flush();
				System.exit(-1);
			}
			if ((TrcParser.traceType.equals("unix") || TrcParser.traceType.equals("windows") || TrcParser.traceType.equals("aat")) && TrcParser.dumpLineWidth != 16)
			{
				System.err.println("dumpLineWidth (-w) must be 16 for traceType unix, windows, or aat!");
				System.err.flush();
				System.exit(-1);
			}
		}
		
		
		// messageParsing
		if (TrcParser.messageParsingGiven)
		{
			if (!TrcParser.traceType.equals("unix") && !TrcParser.traceType.equals("windows"))
			{
				System.err.println("messageParsing only supported for traceType unix, or windows!");
				System.err.flush();
				System.exit(-1);
			}
			
			if (TrcParser.checkDigit(TrcParser.messageParsingStr))
			{
				TrcParser.messageParsingValue = Integer.parseInt(TrcParser.messageParsingStr);
				if (TrcParser.messageParsingValue != 819 && TrcParser.messageParsingValue != 1200 && TrcParser.messageParsingValue != 1208)
				{
					System.err.println("messageParsing (-m) must be either 819, 1200, or 1208!");
					System.err.flush();
					System.exit(-1);
				}
			}
			else
			{
				System.err.println("messageParsing (-m) must be a number!");
				System.err.flush();
				System.exit(-1);
			}
		}
		
		// messageSearch editing
		if (TrcParser.messageSearchGiven)
		{
			if (!TrcParser.traceType.equals("unix") && !TrcParser.traceType.equals("windows") && !TrcParser.traceType.equals("aat"))
			{
				System.err.println("messageSearch only supported for traceType unix, windowsm or aat!");
				System.err.flush();
				System.exit(-1);
			}
			
			if ((TrcParser.messageSearchStrCt > 0 && (TrcParser.messageSearchStr.length() > TrcParser.messageSearchStrMax)) ||
				(TrcParser.messageSearchStrCt > 1 && (TrcParser.messageSearchStr2.length() > TrcParser.messageSearchStrMax)) ||
				(TrcParser.messageSearchStrCt > 2 && (TrcParser.messageSearchStr3.length() > TrcParser.messageSearchStrMax)) ||
				(TrcParser.messageSearchStrCt > 3 && (TrcParser.messageSearchStr4.length() > TrcParser.messageSearchStrMax)) 
			   )
			{
				System.err.println("messageSearch (-s) length can not exceed " + TrcParser.messageSearchStrMax);
				System.err.flush();
				System.exit(-1);
			}
			
			if (TrcParser.messageSearchStrCt > 4)
			{
				System.err.println("messageSearch (-s) can not have more than 4 inputs!");
				System.err.flush();
				System.exit(-1);
			}
			
			if (TrcParser.messageSearchStrCt == 2)
			{
				System.err.println("messageSearch (-s) for multiple string searches must have 3 or 4 inputs and last input must be AND or OR!");
				System.err.flush();
				System.exit(-1);
			}
			
			if (TrcParser.messageSearchStrCt > 2)
			{
				if ((TrcParser.messageSearchStrCt == 3 && (!TrcParser.messageSearchStr3.equals("AND") && !TrcParser.messageSearchStr3.equals("OR"))) ||
					(TrcParser.messageSearchStrCt == 4 && (!TrcParser.messageSearchStr4.equals("AND") && !TrcParser.messageSearchStr4.equals("OR")))	
				   )
				{
					System.err.println("messageSearch (-s) for multiple string searches must have AND or OR as last input!");
					System.err.flush();
					System.exit(-1);
				}

			}

		}
		
		// determine input trace files to parse
		ArrayList<String> inputTrcFiles = new ArrayList<String>();
		
        if (TrcParser.inputFile.substring(0,6).equals("AMQTRC"))
        {
            try {
	        	BufferedReader b = new BufferedReader(new FileReader(TrcParser.inputFile));
	            String readLine = "";
	            while ((readLine = b.readLine()) != null) {
	            	inputTrcFiles.add(readLine);
	            }
	            b.close();
            }
            catch (Exception ex) {
            	System.err.println("Issue when accessing " + TrcParser.inputFile + " file!");
            	ex.printStackTrace();
                for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
                  System.err.println("... Caused by ");
                  t.printStackTrace();
                }
                System.err.flush();
                System.exit(-1);
            }
        }
        else
        {
        	inputTrcFiles.add(inputFile);
        }
        
        // init lastReportedTimeMillis to current time if we are reporting status to user
        if (TrcParser.statusInterval > 0)
        	lastReportedTimeMillis = System.currentTimeMillis();
        
        // create a thread for each input trace file and parse it
        for (int counter = 0; counter < inputTrcFiles.size(); counter++) { 		      
        	// get file length in bytes if we are reporting status to user
        	if (TrcParser.statusInterval > 0)
        	{
        		try {
        			File f = new File(inputTrcFiles.get(counter));
        			TrcParser.totalBytes += f.length();
        		}
                catch (Exception ex) {
                	System.err.println("Exception encountered = " + ex);
                	ex.printStackTrace();
                    for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
                      System.err.println("... Caused by ");
                      t.printStackTrace();
                    }
                    System.err.flush();
                    System.exit(-1);
                }
        	}
        	trcParser.launchThread(counter,inputTrcFiles.get(counter));
        }
        
	}
	
	private void launchThread(int i, String trcFile) {
		Thread processTrcFile = new ProcessTrcFile("Thread"+i,trcFile);
		processTrcFile.start();
	    return;
	}
	
	class ProcessTrcFile extends Thread {

		private String myName;
		private String myTrcFile;
		private Hashtable <String, APITracker> apiht = new Hashtable<String, APITracker>();
		
		public ProcessTrcFile(String threadName, String trcFile) {
			myName = threadName;
			myTrcFile = trcFile;
			return;
		}
		
		public void run() { 
			
			try {
	        	// determine output trace file name and create FileOutputStream
				String outputTrcFile = myTrcFile + TrcParser.outputFileSuffix;
				FileOutputStream outputTrcFileStream = new FileOutputStream(outputTrcFile, false);
				BufferedOutputStream outputTrcBufferedFileStream = new BufferedOutputStream(outputTrcFileStream);
								
				// open the input trace file
	        	BufferedReader b = new BufferedReader(new FileReader(myTrcFile));
	            
	        	HashMap <String, TrcCtl> tchm = new HashMap<String, TrcCtl>();
	        	TrcCtl trcCtl = null;
	        	String readLine = "";
	            String pidTid = null;
	            int lineNmbr=0;
                                
	            // process input trace file
	            while ((readLine = b.readLine()) != null) {

	            	// find pid.tid for our trace line (if applicable)
	            	pidTid = TrcParser.findPidTid(readLine);
	            	
	            	// use NOTFOUND key if not a pid.tid line
	            	if (pidTid == null)
	            		pidTid = "NOTFOUND";
	            	
	            	// get our TrcCtl object for our current pid.tid
	            	trcCtl = tchm.get(pidTid);
	            	
	            	// if not found, create it and add it to our TrcCtl hash map
	            	if (trcCtl == null)
	            	{
	            		trcCtl = new TrcCtl(pidTid);
	            		tchm.put(pidTid, trcCtl);
	            		if(!pidTid.equals("NOTFOUND"))
	            		{
	            			trcCtl.pidTidPos = readLine.indexOf(" " + pidTid);
	            			trcCtl.pidTidPos++;
	            		}
	            		trcCtl.outputTrcFile = outputTrcFile;
	            		trcCtl.outputTrcBufferedFileStream = outputTrcBufferedFileStream;
	            		trcCtl.inputFileCharset = TrcParser.inputFileCharset;
	            		trcCtl.pointerLength = TrcParser.pointerLength;
	            		if (TrcParser.messageParsingGiven)
	            		{
	            			trcCtl.messageParsing = true;
	            			trcCtl.messageParsingValue = TrcParser.messageParsingValue;
	            		}
	            		if (TrcParser.messageSearchGiven)
	            		{	
	            			trcCtl.messageSearch = true;
	            			trcCtl.messageSearchStr = TrcParser.messageSearchStr;
	            			trcCtl.messageSearchStr2 = TrcParser.messageSearchStr2;
	            			trcCtl.messageSearchStr3 = TrcParser.messageSearchStr3;
	            			trcCtl.messageSearchStr4 = TrcParser.messageSearchStr4;
	            			trcCtl.messageSearchStrCt = TrcParser.messageSearchStrCt;
	            			trcCtl.messageSearchStrMax = TrcParser.messageSearchStrMax;
	            		}
	            		if (TrcParser.msg2FileGlobalGiven)
	            		{	
	            			trcCtl.msg2FileGlobal = true;
	            			trcCtl.msg2FileGlobalStr = TrcParser.msg2FileGlobalStr;
	            		}
	            		
	            		if (TrcParser.traceType.equals("aat"))
	            		{
	            			trcCtl.atTracker = new ActivityTraceTracker();
	            			trcCtl.atrTreeMap = new TreeMap<String, ActivityTraceRecord>();
	            		}
	            		
	            	}
	            	
	            	//Special logic to write out Buffer data when we are at the end of the Buffer, but before we write out the next input trace line
	            	if (trcCtl.dataStrcName != null && (trcCtl.dataStrcName.equals("Buffer:") || trcCtl.dataStrcName.equals("aatBuffer:")) && !TrcParser.testForHexLine(trcCtl, readLine) && trcCtl.messageBufferList.size() > 0)
	            	{
	            		trcCtl.hexLine = false;
	            		DataStrcUtil.processDataStrcName(trcCtl);
	            	}

	            	if (TrcParser.traceType.equals("aat"))
	        		{
	        			String aatLine = TrcParser.processAATLine(trcCtl, readLine);
	        			if (aatLine != null)
	        				TrcParser.writeOutputFile(trcCtl, aatLine);
	        		}
	            	
	            	TrcParser.writeOutputFile(trcCtl, readLine);
	            	
	            	lineNmbr++;
	            	trcCtl.lineNmbr = lineNmbr;
	            	
	            	TrcParser.checkForAPI(trcCtl, readLine, this);
	            	
	            	if (trcCtl.dataStrcName != null)
	            	{
	            		TrcParser.processDataStrc(trcCtl, readLine);
	            	}
	            	
	            	if (trcCtl.dataStrcName == null)
	            	{
	            		TrcParser.checkForDataStrc(trcCtl, readLine);
	            		if (trcCtl.dataStrcName != null)
	            			DataStrcUtil.processDataStrcName(trcCtl);
	            	}
	            	
	            	// special logic to handle MQOPEN/MQCLOSE Options that spans many lines between keywords
	            	if (trcCtl.mqOpen || trcCtl.mqClose)
	            	{
	            		trcCtl.mqOpenCloseCt++;
	            		if (trcCtl.mqOpenCloseCt > 100)
	            		{
	            			trcCtl.mqOpen = false;
	            			trcCtl.mqClose = false;
	            			trcCtl.mqOpenCloseCt = 0;
	            		}
	            	}
	            	
	            	// if we are reporting status to the user, add in our string length
	            	if (TrcParser.statusInterval > 0)
	            		TrcParser.checkStatusInterval(readLine.length());
	            	
	            }
	            
	            // Call processDataStrcName one last time to process anything that is outstanding for each pid.tid
	             for (TrcCtl trcCtl2 : tchm.values())
	             {
	            	 if (trcCtl2.dataStrcName != null)
	            	 {
	            		 trcCtl2.hexLine = false;
	            		 DataStrcUtil.processDataStrcName(trcCtl2);
	            	 }
	            	 
	             }
	             
            	 // Write out API report, if requested
	             if (TrcParser.reportStr.equals("API"))
            	 {            	
	            	 TrcParser.printAPIProcess(this, outputTrcBufferedFileStream, outputTrcFile, TrcParser.inputFileCharset);
	            	 
	            	 for (TrcCtl trcCtl2 : tchm.values())
            			 TrcParser.printAPIThread(trcCtl2);
            	 }

	             b.close();
	             
            	 if (TrcParser.traceType.equals("aat"))
            		 TrcParser.printAATReport(trcCtl);
	             
	             // flush out any data still in the output buffer
	             outputTrcBufferedFileStream.flush();
	            
			}
            catch (Exception ex) {
            	System.err.println(myName + " error encountered when processing " + myTrcFile);
            	ex.printStackTrace();
                for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
                  System.err.println("... Caused by ");
                  t.printStackTrace();
                }
                System.err.flush();
                return;
            }
			
		}
	}
	
	private static void parseArgs(String[] args) {
	    try {
	      
	      TrcParser.inputFile = null;
	      TrcParser.fileCharSetGiven = false;
	      TrcParser.traceTypeGiven = false;
	      TrcParser.endiannessGiven = false;
	      TrcParser.constFieldGiven = false;
	      TrcParser.constValueGiven = false;
	      TrcParser.pointerLengthGiven = false;
	      TrcParser.msg2FileGlobalGiven = false;
	      TrcParser.messageParsingGiven = false;
	      TrcParser.messageSearchGiven = false;
	      TrcParser.dumpLineWidthGiven = false;
	      TrcParser.reportGiven = false;
	      TrcParser.printCharGiven = false;
	      
	      int length = args.length;
	      if (length == 0) {
	        throw new IllegalArgumentException("No arguments! Mandatory arguments must be specified.");
	      }
	      if ((length % 2) != 0) {
	        throw new IllegalArgumentException("Incorrect number of arguments!");
	      }

	      int i = 0;

	      while (i < length) {
	        if ((args[i]).charAt(0) != '-') {
	          throw new IllegalArgumentException("Expected a '-' character next: " + args[i]);
	        }

	        char opt = (args[i]).toLowerCase().charAt(1);

	        switch (opt) {
	          case 'a' :
	        	TrcParser.printCharStr = args[++i];
	        	TrcParser.printCharGiven = true;
	        	break;
	          case 'b' :
	        	TrcParser.statusInterval = Long.parseLong(args[++i]) * 1000;
	        	break;
	          case 'c' :
	        	TrcParser.inputFileCharset = args[++i];
	        	TrcParser.fileCharSetGiven = true;
	            break;  
	          case 'e' :
	        	TrcParser.endianness = args[++i];
	        	TrcParser.endiannessGiven = true;
	            break;
	          case 'f' :
		        TrcParser.constField = args[++i];
		        TrcParser.constFieldGiven = true;
		        break;
	          case 'g' :
			    TrcParser.msg2FileGlobalStr = args[++i];
			    TrcParser.msg2FileGlobalGiven = true;
			    break;
	          case 'i' :
	        	TrcParser.inputFile = args[++i];
	            break;
	          case 'm' :
	        	TrcParser.messageParsingStr = args[++i];
	        	TrcParser.messageParsingGiven = true;
	            break;
	          case 'o' :
	        	TrcParser.outputFileSuffix = args[++i];
	            break;
	          case 'p' :
	        	TrcParser.pointerLengthGiven = true;
	        	TrcParser.pointerLengthStr = args[++i];
	            break;
	          case 'r' :
	        	TrcParser.reportGiven = true;
	        	TrcParser.reportStr = args[++i];
	            break;
	          case 's' :
	        	TrcParser.messageSearchStrCt++;
	        	switch (TrcParser.messageSearchStrCt) {
	              case 1 :
	            	TrcParser.messageSearchStr = args[++i];
	            	break;
	              case 2 :
	            	TrcParser.messageSearchStr2 = args[++i];
		            break;
	              case 3 :
		            TrcParser.messageSearchStr3 = args[++i];
			        break;
	              case 4 :
		            TrcParser.messageSearchStr4 = args[++i];
			        break;
			      default :
			    	i++;
			    	break;
	        	}
	        	TrcParser.messageSearchGiven = true;
	        	break;
	          case 't' :
	        	TrcParser.traceType = args[++i];
	        	TrcParser.traceTypeGiven = true;
	        	break;
	          case 'v' :
		        TrcParser.constValueStr = args[++i];
		        TrcParser.constValueGiven = true;
		        break;
	          case 'w' :
	        	TrcParser.dumpLineWidthStr = args[++i];
	        	TrcParser.dumpLineWidthGiven = true;
	        	break;
	          case 'x' :
	        	TrcParser.dumpLinePrefix = args[++i];
	        	break;
	          default : {
	            throw new IllegalArgumentException("Unknown argument: " + opt);
	          }
	        }

	        ++i;
	      }

	    }
	    catch (Exception e) {
	      System.err.println(e.getMessage());
	      printUsage();
	      System.exit(-1);
	    }
	    return;
	}
	
	private static void printUsage() {
	    System.err.println("\nUsage (consult mqtrcfrmt manual for more details):");
	    System.err
	        .println("mqtrcfrmt -i inputFile [-a printChar -b statusInterval -c inputFileCharset -e endianness -f constantField (enter ? to list supported fields) -g msg2FileGlobal -m messageParsing -o outputFileSuffix -p pointerLength -r report -s messageSearch -t traceType -v constantFieldValue -w dumpLineWidth -x dumpLinePrefix]");
	    return;
	}		  
	
	public static synchronized void checkStatusInterval(int byteCt)
	{
		long currentTimeMillis = System.currentTimeMillis();
		
		try {
			TrcParser.bytesProcessed += byteCt;
			
			if (currentTimeMillis > (TrcParser.lastReportedTimeMillis + TrcParser.statusInterval))
			{
				TrcParser.lastReportedTimeMillis = currentTimeMillis;
				System.out.println("mqtrcfrmt is " + 
				                  ((TrcParser.bytesProcessed * 100) / TrcParser.totalBytes) + 
				                  " percent done.");
			}			
		}
		catch (Exception ex) {
        	System.err.println("Exception encountered = " + ex);
        	ex.printStackTrace();
            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
              System.err.println("... Caused by ");
              t.printStackTrace();
            }
            System.err.flush();
            System.exit(-1);
		}

	}
	
	public static String findPidTid(String line) {
		String pidTid = null;
		
		// if not a unix or windows strmqtrc trace, there will be no pid.tid
		if (!TrcParser.traceType.equals("unix") && !TrcParser.traceType.equals("windows"))
			return pidTid;
        		
		// first look for timestamp. if found, the pid.tid should be the next token.
		// if pid.tid is found at next token, return it as pid.tid.
		int y=0;
		String [] result = line.split("\\s+");
		for (int x=0; x < result.length; x++)
		{
			y = result[x].indexOf(":");
			if (y > 0 && y != result[x].lastIndexOf(":"))
				if (x+1 < result.length && result[x+1].indexOf(".") > 0)
					return result[x+1];
		}
		
		return pidTid;
				
	}
	
	public static String processAATLine(TrcCtl trcCtl, String line) {
		
		String outputString = null;
		
        try {
        	
			// at end of MQI Operation record
			if (trcCtl.atTracker.inMQIOpr && line.length() >= 2 && !line.substring(0,2).equals("  "))
			{
				StringBuilder builder = new StringBuilder();
				builder.append("1LS= Rec(");
				builder.append(trcCtl.atTracker.recordNumber);
				builder.append(") applName(");
				if (trcCtl.atTracker.applName != null)
					builder.append(trcCtl.atTracker.applName);
				builder.append(") userId(");
				if (trcCtl.atTracker.userId != null)
					builder.append(trcCtl.atTracker.userId);
				builder.append(") Pid(");
				if (trcCtl.atTracker.pid != null)
					builder.append(trcCtl.atTracker.pid);
				builder.append(") Tid(");
				if (trcCtl.atTracker.tid != null)
					builder.append(trcCtl.atTracker.tid);
				builder.append(") Date(");
				if (trcCtl.atTracker.date != null)
					builder.append(trcCtl.atTracker.date);
				builder.append(") Time(");
				if (trcCtl.atTracker.time != null)
					builder.append(trcCtl.atTracker.time);
				if (trcCtl.atTracker.highResTime != null)
				{
					builder.append(") HighResTime(");
					builder.append(trcCtl.atTracker.highResTime);
				}
				builder.append(") Opr(");
				if (trcCtl.atTracker.opr != null)
					builder.append(trcCtl.atTracker.opr);
				builder.append(") RC(");
				if (trcCtl.atTracker.reason != null)
					builder.append(trcCtl.atTracker.reason);
				builder.append(") Chl(");
				if (trcCtl.atTracker.channel != null)
					builder.append(trcCtl.atTracker.channel);
				builder.append(") CnId(");
				if (trcCtl.atTracker.connId != null)
					builder.append(trcCtl.atTracker.connId);
				builder.append(") HObj(");
				if (trcCtl.atTracker.hObj != null)
					builder.append(trcCtl.atTracker.hObj);
				builder.append(") Obj(");
				if (trcCtl.atTracker.objName != null)
					builder.append(trcCtl.atTracker.objName);
				builder.append(")");
				
				// We assume that when we have hit the line immediately following an MQI Operation block,  
				// there will be no need to return another string in the lower logic of this method.  
				// Therefore, just set the outputString here and let it return in the lower logic.
				outputString = builder.toString();
				
				// Store this data on our tree map for ActivityTraceRecord
				builder = new StringBuilder();
				builder.append(trcCtl.atTracker.qmgrName);
				builder.append(trcCtl.atTracker.applName);
				builder.append(trcCtl.atTracker.userId);
				builder.append(trcCtl.atTracker.pid);
				builder.append(trcCtl.atTracker.channel);
				builder.append(trcCtl.atTracker.connId);
				String atrKey = builder.toString();
				ActivityTraceRecord atr = trcCtl.atrTreeMap.get(atrKey);
				if (atr == null)
				{
					atr = new ActivityTraceRecord();
					atr.qmgrName = trcCtl.atTracker.qmgrName;
					atr.applName = trcCtl.atTracker.applName;
					atr.userId = trcCtl.atTracker.userId;
					atr.pid = trcCtl.atTracker.pid;
					atr.channel = trcCtl.atTracker.channel;
					atr.connId = trcCtl.atTracker.connId;
					trcCtl.atrTreeMap.put(atrKey, atr);
				}
				builder = new StringBuilder();
				builder.append(trcCtl.atTracker.opr);
				builder.append(trcCtl.atTracker.objType);
				builder.append(trcCtl.atTracker.objName);
				builder.append(trcCtl.atTracker.persistence);
				String atmKey = builder.toString();
				ActivityTraceMQI atm = atr.mqiTreeMap.get(atmKey);
				if (atm == null)
				{
					atm = new ActivityTraceMQI();
					atm.opr = trcCtl.atTracker.opr;
					atm.objectType = trcCtl.atTracker.objType;
					atm.objectName = trcCtl.atTracker.objName;
					atm.persistence = trcCtl.atTracker.persistence;
					atr.mqiTreeMap.put(atmKey, atm);
				}
				atm.oprCt++;
				atm.oprTotalDuration += trcCtl.atTracker.oprDuration;
				atm.msgTotalLength += trcCtl.atTracker.msgLength;
				atm.reasonTreeSet.add(trcCtl.atTracker.reason);
				
				// reset values on our atTracker at MQI level
				trcCtl.atTracker.inMQIOpr = false;
				trcCtl.atTracker.tid = null;
				trcCtl.atTracker.date = null;
				trcCtl.atTracker.time = null;
				trcCtl.atTracker.highResTime = null;
				trcCtl.atTracker.opr = null;
				trcCtl.atTracker.reason = null;
				trcCtl.atTracker.hObj = null;
				trcCtl.atTracker.objName = null;
				trcCtl.atTracker.objType = null;
				trcCtl.atTracker.persistence = null;
				trcCtl.atTracker.oprDuration = 0;
				trcCtl.atTracker.msgLength = 0;
			}
			
			// Activity Trace record beginning 
			if (line.equals("MonitoringType: MQI Activity Trace"))
			{
				trcCtl.atTracker.qmgrName = null;
				trcCtl.atTracker.connId = null;
				trcCtl.atTracker.applName = null;
				trcCtl.atTracker.userId = null;
				trcCtl.atTracker.pid = null;
				trcCtl.atTracker.channel = null;
				trcCtl.atTracker.recordNumber++;
				outputString = "Rec(" + trcCtl.atTracker.recordNumber + ")";
				return outputString;
			}
			
			// queueManager
			if (line.length() > 14 && line.substring(0,14).equals("QueueManager: "))
			{
				trcCtl.atTracker.qmgrName = line.substring(14);
				return outputString;
			}
			
			// connection id 
			if (trcCtl.atTracker.connIdCtr > 0)
				trcCtl.atTracker.connIdCtr++;
			
			if (trcCtl.atTracker.connIdCtr == 3)
			{
				trcCtl.atTracker.connIdCtr = 0;
				if (line.length() >= 30)
					trcCtl.atTracker.connId = line.substring(11,15) + line.substring(16,20) + line.substring(21,25) + line.substring(26,30); 
			}
			
			if (line.equals("Correl_id:"))
			{
				trcCtl.atTracker.connIdCtr = 1;
				return outputString;			
			}
			
			// applName
			if (line.length() > 17 && line.substring(0,17).equals("ApplicationName: "))
			{
				trcCtl.atTracker.applName = line.substring(17);
				return outputString;
			}
			
			// userId
			if (line.length() > 8 && line.substring(0,8).equals("UserId: "))
			{
				trcCtl.atTracker.userId = line.substring(8);
				return outputString;
			}
			
			// pid
			if (line.length() > 16 && line.substring(0,16).equals("ApplicationPid: "))
			{
				trcCtl.atTracker.pid = line.substring(16);
				return outputString;
			}
			
			// channel
			if (line.length() > 14 && line.substring(0,14).equals("Channel Name: "))
			{
				trcCtl.atTracker.channel = line.substring(14);
				return outputString;
			}
			
			// MQI Operation block
			if (line.length() >= 14 && line.substring(0,14).equals("MQI Operation:"))
			{
				trcCtl.atTracker.inMQIOpr = true;
				return outputString;
			}
			
			// tid
			if (line.length() > 18 && line.substring(0,18).equals("  ApplicationTid: "))
			{
				trcCtl.atTracker.tid = line.substring(18);
				return outputString;
			}
			
			// date
			if (line.length() > 17 && line.substring(0,17).equals("  OperationDate: "))
			{
				trcCtl.atTracker.date = line.substring(17);
				return outputString;
			}
			
			// time
			if (line.length() > 17 && line.substring(0,17).equals("  OperationTime: "))
			{
				trcCtl.atTracker.time = line.substring(17);
				return outputString;
			}
			
			// highResTime
			if (line.length() > 17 && line.substring(0,17).equals("  High Res Time: "))
			{
				trcCtl.atTracker.highResTime = line.substring(17);
				return outputString;
			}
			
			// operation id
			if (line.length() > 16 && line.substring(0,16).equals("  Operation Id: "))
			{
				trcCtl.atTracker.opr = line.substring(16);
				return outputString;
			}
			
			// reason code
			if (line.length() > 15 && line.substring(0,15).equals("  Reason Code: "))
			{
				trcCtl.atTracker.reason = line.substring(15);
				return outputString;
			}
			
			// hObj
			if (line.length() > 8 && line.substring(0,8).equals("  Hobj: "))
			{
				trcCtl.atTracker.hObj = line.substring(8);
				return outputString;
			}
			
			// objectName
			if (line.length() > 15 && line.substring(0,15).equals("  Object_name: "))
			{
				trcCtl.atTracker.objName = line.substring(15);
				return outputString;
			}
			
			// objectType
			if (line.length() > 15 && line.substring(0,15).equals("  Object_type: "))
			{
				trcCtl.atTracker.objType = line.substring(15);
				return outputString;
			}
			
			// persistence
			if (line.length() > 15 && line.substring(0,15).equals("  Persistence: "))
			{
				trcCtl.atTracker.persistence = line.substring(15);
				if (trcCtl.atTracker.persistence.length() > 1)
					trcCtl.atTracker.persistence = null;
				return outputString;
			}
			
			// oprDuration
			if (line.length() > 27 && line.substring(0,27).equals("  QMgr Operation Duration: "))
			{
				trcCtl.atTracker.oprDuration = Long.parseLong(line.substring(27));
				return outputString;
			}
			
			// messageLength
			if (line.length() > 14 && line.substring(0,14).equals("  Msg length: "))
			{
				trcCtl.atTracker.msgLength = Long.parseLong(line.substring(14));
				return outputString;
			}

        } catch (Exception ex) {
        	System.err.println("Exception encountered in Activity Trace parsing logic = " + ex);
        	ex.printStackTrace();
            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
              System.err.println("... Caused by ");
              t.printStackTrace();
            }
            System.err.flush();
            System.exit(-1);
        }
		
		return outputString;
		
	}
	
	public static void printAATReport(TrcCtl trcCtl) {
		
		try {
			
			TrcParser.writeOutputFile(trcCtl, "");
			TrcParser.writeOutputFile(trcCtl, "*****************************************************");
			TrcParser.writeOutputFile(trcCtl, "* mqtrcfrmt AAT Connection Summary Report           *");
			TrcParser.writeOutputFile(trcCtl, "*                                                   *");
			TrcParser.writeOutputFile(trcCtl, "* NOTE: Report items are sorted descending and by   *");
			TrcParser.writeOutputFile(trcCtl, "* qmgrName, applName, userId, pid, channel, connId. *");
			TrcParser.writeOutputFile(trcCtl, "*****************************************************");
			TrcParser.writeOutputFile(trcCtl, "");
			
			StringBuilder builder = null;
		
			for (ActivityTraceRecord atr : trcCtl.atrTreeMap.values())
			{
				builder = new StringBuilder();
				builder.append("qmgrName(");
				builder.append(atr.qmgrName);
				builder.append(") applName(");
				builder.append(atr.applName);
				builder.append(") userId(");
				builder.append(atr.userId);
				builder.append(") pid(");
				builder.append(atr.pid);
				builder.append(") channel(");
				if (atr.channel != null)
					builder.append(atr.channel);
				builder.append(") connId(");
				builder.append(atr.connId);
				builder.append("):");
				TrcParser.writeOutputFile(trcCtl, builder.toString());
				for (ActivityTraceMQI atm : atr.mqiTreeMap.values())
				{
					builder = new StringBuilder();
					builder.append("Opr(");
					builder.append(atm.opr);
					builder.append(") OprCt(");
					builder.append(atm.oprCt);
					builder.append(")");
					if (atm.oprTotalDuration > 0)
					{
						builder.append(" avgOprDuration(");
						builder.append(atm.oprTotalDuration / atm.oprCt);
						builder.append(")");
					}
					if (atm.objectType != null)
					{
						builder.append(" ObjType(");
						builder.append(atm.objectType);
						builder.append(")");
					}
					// ignore objectNames that begin with a space
					if (atm.objectName != null)
					{
						String objectName = atm.objectName; 
						// Change objectName to empty string if it starts with a space.
						// This corrects for scenarios where AAT returns some object names with long
						// spaces followed by a delimiter character.
						if (atm.objectName.length() >= 2 && atm.objectName.substring(0,2).equals("' "))
							objectName = "";
						builder.append(" ObjName(");
						builder.append(objectName);
						builder.append(")");
					}
					if (atm.persistence != null)
					{
						builder.append(" Prst(");
						builder.append(atm.persistence);
						builder.append(")");
					}
					if (atm.msgTotalLength > 0)
					{
						builder.append(" msgTotalLength(");
						builder.append(atm.msgTotalLength);
						builder.append(")");
					}
					builder.append(" reasonCds=");
					boolean firstTime = true;
					for (String reason : atm.reasonTreeSet)
					{
						if (firstTime)
							firstTime = false;
						else
							builder.append(",");
						builder.append(reason);
					}
					TrcParser.writeOutputFile(trcCtl, builder.toString());
				}
				TrcParser.writeOutputFile(trcCtl, "");
			}
        } catch (Exception ex) {
        	System.err.println("Exception encountered in Activity Trace report logic = " + ex);
        	ex.printStackTrace();
            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
              System.err.println("... Caused by ");
              t.printStackTrace();
            }
            System.err.flush();
            System.exit(-1);
        }

	}
	
	public static void checkForAPI(TrcCtl trcCtl, String line, ProcessTrcFile ptf) {
		
		int y=0;
		int z=0;
		String [] result = line.split("\\s+");
		String prevStr = "";
		String holdMqAPITimestamp = null;
		
		for (int x=0; x < result.length; x++)
		{
			// check for API timestamp
			if (TrcParser.reportStr.equals("API"))
			{
				y = result[x].indexOf(":");
				z = result[x].lastIndexOf(":");
				if (y > 0 && y != z && result[x].length() == 15)
					if (result[x].substring(2,3).equals(":") && result[x].substring(5,6).equals(":") && result[x].substring(8,9).equals(".") &&
							TrcParser.checkDigit(result[x].substring(0,2)) && TrcParser.checkDigit(result[x].substring(3,5)) &&
							TrcParser.checkDigit(result[x].substring(6,8)) && TrcParser.checkDigit(result[x].substring(9)))
						holdMqAPITimestamp = result[x];
			}
			
			// check for MQ API (e.g. MQPUT >>)
			if (result[x].equals("<<") || result[x].equals(">>"))
			{
				if (prevStr.length() > 2 && prevStr.substring(0,2).equals("MQ"))
				{
					trcCtl.mqPrevAPI = trcCtl.mqAPI;
					trcCtl.mqPrevAPIType = trcCtl.mqAPIType;
					trcCtl.mqPrevAPITimestamp = trcCtl.mqAPITimestamp;
					trcCtl.mqAPI = prevStr;
					trcCtl.mqAPIType = result[x];
					trcCtl.mqAPITimestamp = holdMqAPITimestamp;
					
					if (TrcParser.reportStr.equals("API") && result[x].equals("<<") && trcCtl.mqAPI.equals(trcCtl.mqPrevAPI) &&
							trcCtl.mqAPITimestamp != null && trcCtl.mqPrevAPITimestamp != null)
						TrcParser.processAPI(trcCtl, ptf);
						
				}
			}
			prevStr = result[x];
		}
		
	}
	
	public static void processAPI(TrcCtl trcCtl, ProcessTrcFile ptf) {

		double elapsedTime = TrcParser.getElapsedTime(trcCtl);
		
		// ptf.apiht tracks API data at process level
		APITracker apitPid = ptf.apiht.get(trcCtl.mqAPI);
		
		if (apitPid == null)
    	{
    		apitPid = new APITracker(trcCtl.mqAPI);
    		ptf.apiht.put(trcCtl.mqAPI, apitPid);
    	}
		
		apitPid.totAPITime += elapsedTime;
    	apitPid.totAPICalls++;
		
		// trcCtl.apihm tracks API data at thread level
    	APITracker apitTid = trcCtl.apihm.get(trcCtl.mqAPI);
    	
    	if (apitTid == null)
    	{
    		apitTid = new APITracker(trcCtl.mqAPI);
    		trcCtl.apihm.put(trcCtl.mqAPI, apitTid);
    	}
    	
    	apitTid.totAPITime += elapsedTime;
    	apitTid.totAPICalls++;
    	
	}
	
	public static void printAPIProcess(ProcessTrcFile ptf, BufferedOutputStream outputTrcBufferedFileStream, String outputTrcFile, String inputFileCharset) {
		
		TrcParser.writeOutputBufferedFileStream(outputTrcBufferedFileStream, outputTrcFile, inputFileCharset, "API Report For Process:");
		for (String key : ptf.apiht.keySet())
		{
			APITracker apit = ptf.apiht.get(key);
			TrcParser.writeOutputBufferedFileStream(outputTrcBufferedFileStream, outputTrcFile, inputFileCharset, "API=" + apit.mqAPI + " totalCalls=" + apit.totAPICalls + " avgRespTimeSeconds=" + apit.totAPITime / apit.totAPICalls);
		}
		TrcParser.writeOutputBufferedFileStream(outputTrcBufferedFileStream, outputTrcFile, inputFileCharset, "");
		
	}
	
	public static void printAPIThread(TrcCtl trcCtl) {
		if (!trcCtl.pidTid.equals("NOTFOUND"))
		{
			TrcParser.writeOutputFile(trcCtl, "API Report For " + trcCtl.pidTid + ":");
			for (String key : trcCtl.apihm.keySet())
			{
				APITracker apit = trcCtl.apihm.get(key);
				TrcParser.writeOutputFile(trcCtl, "API=" + apit.mqAPI + " totalCalls=" + apit.totAPICalls + " avgRespTimeSeconds=" + apit.totAPITime / apit.totAPICalls);
			}
			TrcParser.writeOutputFile(trcCtl, "");
		}
	}
	
	public static double getElapsedTime(TrcCtl trcCtl) {
	   double begHour = 0;
	   double endHour = 0;
	   double min = 0;
	   double sec = 0;
	   double subsec = 0;
	   double begTimeSeconds = 0;
	   double endTimeSeconds = 0;
	   double elapsedTime = 0;
	   
	   begHour = Double.parseDouble(trcCtl.mqPrevAPITimestamp.substring(0,2)) * 3600;
	   min = Double.parseDouble(trcCtl.mqPrevAPITimestamp.substring(3,5)) * 60;
	   sec = Double.parseDouble(trcCtl.mqPrevAPITimestamp.substring(6,8));
	   subsec = Double.parseDouble(trcCtl.mqPrevAPITimestamp.substring(9)) * .000001;
	   begTimeSeconds = begHour + min + sec + subsec;
	   
	   endHour = Double.parseDouble(trcCtl.mqAPITimestamp.substring(0,2)) * 3600;
	   min = Double.parseDouble(trcCtl.mqAPITimestamp.substring(3,5)) * 60;
	   sec = Double.parseDouble(trcCtl.mqAPITimestamp.substring(6,8));
	   subsec = Double.parseDouble(trcCtl.mqAPITimestamp.substring(9)) * .000001;
	   endTimeSeconds = endHour + min + sec + subsec;
	   
	   // Add in 24 hours (86,400 seconds) to endTimeSeconds, if we have rolled past midnight.
	   // We will assume we are dealing with consecutive days.
	   if (endHour < begHour)
		   endTimeSeconds = endTimeSeconds + 86400;
	   
	   elapsedTime = endTimeSeconds - begTimeSeconds;
	   return elapsedTime;
	}
	
	public static void checkForDataStrc(TrcCtl trcCtl, String line) {
		
		// Check for data structures that we parse for strmqtrc or aat format
		int aatMessageCt = -1;
		String [] result = line.split("\\s+");
		for (int x=0; x < result.length; x++)
		{
			if (TrcParser.traceType.equals("unix") || TrcParser.traceType.equals("windows"))
			{
				switch (result[x]) {
					case "Bufferlength:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "Compcode:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "ConnectOpts:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "Datalength:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "Hconn:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "Hobj:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "Hmsg:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "MQOPEN" :
						trcCtl.mqOpen = true;
						trcCtl.mqClose = false;
						break;
					case "MQCLOSE" :
						trcCtl.mqClose = true;
						trcCtl.mqOpen = false;
						break;
					case "Options:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "Reason:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					case "Buffer:" :
						if (!TrcParser.testForNull(result, x+1))
							trcCtl.dataStrcName = result[x];
						break;
					default :
						break;
				}
				
				// Check for msg2File- keyword
				if (result[x].length() > 9 && result[x].substring(0, 9).equals("msg2File-"))
				{
					trcCtl.msg2FileStr = result[x].substring(9);
					trcCtl.msg2File = true;
				}
				
			}
			
			if (TrcParser.traceType.equals("aat"))
			{
				switch (result[x]) {
					case "Message" :
						aatMessageCt=x;
						break;
					case "Data:" :
						if (aatMessageCt > -1 && (aatMessageCt + 1 == x))
						{
							trcCtl.dataStrcName = "aatBuffer:";
						}
						break;
					default :
						break;
				}
				
				// Check for msg2File- keyword
				if (result[x].length() > 9 && result[x].substring(0, 9).equals("msg2File-"))
				{
					trcCtl.msg2FileStr = result[x].substring(9);
					trcCtl.msg2File = true;
				}
				
			}

			if (trcCtl.dataStrcName != null)
				break;
						
		}
		
		TrcParser.parseForHexLine(trcCtl, line);
				
		// If we did not find a data structure and we did find a hex line, check to see if this hex line
		// contains a data structure.
		if (trcCtl.dataStrcName == null && trcCtl.dsBufferOffset > 0)
		{
			TrcParser.checkForDataStrcInHexLine(trcCtl);
			if (trcCtl.dataStrcName == null)
				trcCtl.dsBufferOffset = 0;
			else
				trcCtl.indent = trcCtl.tmpIndent;
		}
	}
	
	public static boolean testForNull(String [] line, int pos) {
		
		if (pos < line.length)
		{
			if (line[pos].equalsIgnoreCase("NULL"))
				return true;
		}
		
		return false;
	}
	
	public static void checkForDataStrcInHexLine(TrcCtl trcCtl) {
		try 
		{
			// Check first 4 bytes for eye catcher and next 4 bytes for version
			if (trcCtl.dsBufferOffset > 7)
			{
				String eyeCatcher = new String(trcCtl.dsBuffer, 0, 4, TrcParser.inputFileCharset);
				switch (eyeCatcher) {
				case "CBC " :
					trcCtl.dataStrcName = "CBC";
					break;
				case "CBD " :
					trcCtl.dataStrcName = "CBD";
					break;
				case "CMHO" :
					trcCtl.dataStrcName = "CMHO";
					break;
				case "CTLO" :
					trcCtl.dataStrcName = "CTLO";
					break;
				case "DMHO" :
					trcCtl.dataStrcName = "DMHO";
					break;
				case "DMPO" :
					trcCtl.dataStrcName = "DMPO";
					break;
				case "GMO " :
					trcCtl.dataStrcName = "GMO";
					break;
				case "IMPO" :
					trcCtl.dataStrcName = "IMPO";
					break;
				case "MD  " :
					trcCtl.dataStrcName = "MD";
					break;
				case "OD  " :
					trcCtl.dataStrcName = "OD";
					break;
				case "PD  " :
					trcCtl.dataStrcName = "PD";
					break;
				case "PMO " :
					trcCtl.dataStrcName = "PMO";
					break;
				case "SD  " :
					trcCtl.dataStrcName = "SD";
					break;
				case "SMPO" :
					trcCtl.dataStrcName = "SMPO";
					break;
				case "STAT" :
					trcCtl.dataStrcName = "STAT";
					break;
				default :
				}
				
				if (trcCtl.dataStrcName != null)
				{
					boolean validVersion = true;
					byte holdVersion = 0;
					if (TrcParser.endianness.equals("little"))
					{
						if (trcCtl.dsBuffer[4] == 0 || trcCtl.dsBuffer[4] > 100)
							validVersion = false;
						else
							holdVersion = trcCtl.dsBuffer[4];
						
						if (trcCtl.dsBuffer[5] != 0)
							validVersion = false;
						
						if (trcCtl.dsBuffer[6] != 0)
							validVersion = false;
						
						if (trcCtl.dsBuffer[7] != 0)
							validVersion = false;
					}
					else
					{
						if (trcCtl.dsBuffer[4] != 0)
							validVersion = false;
						
						if (trcCtl.dsBuffer[5] != 0)
							validVersion = false;
						
						if (trcCtl.dsBuffer[6] != 0)
							validVersion = false;
						
						if (trcCtl.dsBuffer[7] == 0 || trcCtl.dsBuffer[7] > 100)
							validVersion = false;
						else
							holdVersion = trcCtl.dsBuffer[7];
					}
					
					if (!validVersion)
						trcCtl.dataStrcName = null;
					else
					{
						trcCtl.dataStrcVersion = (int) holdVersion;
						DataStrcUtil.getDataStrcSize(trcCtl);
					}
				}
			}	
		}
		catch (Exception ex) {
			System.err.println("Error encountered when processing eyecatcher for output file " + trcCtl.outputTrcFile);
        	ex.printStackTrace();
            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
              System.err.println("... Caused by ");
              t.printStackTrace();
            }
            System.err.flush();
            System.exit(-1);
		}
		
	}
	
		
	public static boolean testForHexLine(TrcCtl trcCtl, String line) {
		
		boolean hexLine = false;
		
		String [] result = line.split("\\s+");
		
		for (int x=0; x < result.length; x++)
		{
			if (TrcParser.dumpLinePrefix.length() <= result[x].length() && 
					result[x].substring(0, TrcParser.dumpLinePrefix.length()).equals(TrcParser.dumpLinePrefix))
				{	
					hexLine = true;
				}
		}
		
		return hexLine;
		
	}
	
	public static void parseForHexLine(TrcCtl trcCtl, String line) {

		// if we could potentially exceed our dsBufferMax, just return
		if (trcCtl.dsBufferOffset + TrcParser.dumpLineWidth > trcCtl.dsBufferMax)
			return;
		
		trcCtl.bytesParsed = 0;
		boolean hexLine = false;
		String [] result = line.split("\\s+");
		int endPos = trcCtl.dsBufferOffset + TrcParser.dumpLineWidth;		

		boolean stopCd=false;
		
		for (int x=0; x < result.length && trcCtl.dsBufferOffset < endPos && !stopCd; x++)
		{
			// If dump line token was found, assume next tokens are a string of bytes and convert them into bytes 
			// in the trcCtl.dsBuffer byte array. Also, limit amount of bytes read to the TrcParser.dumpLineWidth.
			if (hexLine)
			{
				int len = result[x].length();
				
				// If we encounter a result that begins with | or : or ', we stop processing.
				// This handles the condition found in strmqtrc traces or amqsact output to not read past the hex bytes
				if (len > 0 && (result[x].substring(0, 1).equals("|") || result[x].substring(0, 1).equals(":") || result[x].substring(0, 1).equals("'")))
					stopCd=true;
				
				if (len % 2 == 0 && TrcParser.checkLetterOrDigit(result[x]) && !stopCd)
				{
					// This check is needed to remove odd non-trace bytes that were added around MQ 9.1 that appear
					// after the dump prefix but before the 16 bytes of the trace data for Windows strmqtrc.
					if (TrcParser.traceType.equals("windows") && len != 2)
					{
						continue;
					}
					else
					{
						trcCtl.bytesParsed = (short) (trcCtl.bytesParsed + (len / 2)); 
						
						for (int i = 0; i < len && trcCtl.dsBufferOffset < endPos; i += 2) {
							trcCtl.dsBuffer[trcCtl.dsBufferOffset] = (byte) ((Character.digit(result[x].charAt(i), 16) << 4)
									+ Character.digit(result[x].charAt(i+1), 16));
							trcCtl.dsBufferOffset++;
							
							if (trcCtl.dataStrcName != null && (trcCtl.dataStrcName.equals("Buffer:") || trcCtl.dataStrcName.equals("aatBuffer:")))
							{
								// Do not add to messageBuffer if this is a duplicate line.  The duplicate line logic in 
								// processDataStrc will add it to the messageBuffer.
								if (!TrcParser.checkForDupLine(line))
								{
									if (TrcParser.messageParsingGiven || TrcParser.messageSearchGiven || trcCtl.msg2File || trcCtl.msg2FileGlobal)
										MessageBufferUtil.messageBufferAddByte(trcCtl);
								
									if (TrcParser.messageParsingGiven)
										MessageParsingUtil.messageParsing(trcCtl, TrcParser.messageParsingValue, TrcParser.endianness);
								}
							}
						}
					}
				}
					
			}
			
			// If dump line prefix token (e.g. starts with 0x) is found, set hexLine to true 
			if (TrcParser.dumpLinePrefix.length() <= result[x].length() && 
				result[x].substring(0, TrcParser.dumpLinePrefix.length()).equals(TrcParser.dumpLinePrefix) && !stopCd)
			{	
				hexLine = true;
				if (x==0)
					trcCtl.tmpIndent = 0;
				else
				{
					trcCtl.tmpIndent = line.indexOf(" " + TrcParser.dumpLinePrefix);
					trcCtl.tmpIndent++;
				}
			}
		}
		
	}
	
	public static boolean checkLetterOrDigit (String str1) {
		boolean valid = true;
		for (char c : str1.toCharArray()) { 
		    if(!Character.isLetterOrDigit(c))
		    {
		        valid = false;
		        break;
		    }
		}
		return valid;
	}
	
	public static boolean checkDigit (String str1) {
		boolean valid = true;
		for (char c : str1.toCharArray()) { 
		    if(!Character.isDigit(c))
		    {
		        valid = false;
		        break;
		    }
		}
		return valid;
	}
	
	public static void processDataStrc(TrcCtl trcCtl, String line) {

		boolean badHexLine = false;
		int holdBufferOffset = trcCtl.dsBufferOffset;
		TrcParser.parseForHexLine(trcCtl, line);
		
		// if we encoutered a hex line, it was not full width, and we are under our dataStrcSize, we will assume this is a poorly
		// formatted data strucutre and potentially set for reset.
		if (trcCtl.bytesParsed > 0 && trcCtl.bytesParsed < TrcParser.dumpLineWidth && trcCtl.dsBufferOffset < trcCtl.dataStrcSize)
			badHexLine = true;
		
		// Process duplicate lines reference for strmqtrc format
		// NOTE: we only support duplicate lines up to 99 for strmqtrc data structures that we parse
		if (TrcParser.traceType.equals("windows") || TrcParser.traceType.equals("unix"))
		{
			
			int x = line.indexOf("lines suppressed, same as above");
			
			if (x == -1)
				x = line.indexOf("Duplicate Lines =");
			
			if (x > 10)
			{
                badHexLine = false;
				int dupLines = 0;
				if (line.charAt(x-1) == ' ' && Character.isDigit(line.charAt(x-2)) && line.charAt(x-3) == ' ')
					dupLines = Integer.parseInt(line.substring(x-2,x-1));

				if (line.charAt(x-1) == ' ' && Character.isDigit(line.charAt(x-2)) && Character.isDigit(line.charAt(x-3)) && line.charAt(x-4) == ' ')
					dupLines = Integer.parseInt(line.substring(x-3,x-1));
				
				if (line.charAt(x-1) == ' ' && Character.isDigit(line.charAt(x-2)) && Character.isDigit(line.charAt(x-3)) && Character.isDigit(line.charAt(x-4)) && line.charAt(x-5) == ' ')
					dupLines = Integer.parseInt(line.substring(x-4,x-1));
				
				if (dupLines == 0)
				{
					System.err.println("Error encountered when processing strmqtrc duplicate line syntax for output file " + trcCtl.outputTrcFile + " at lineNmbr: " + trcCtl.lineNmbr);
					System.err.println("NOTE: Only duplicate lines up to 999 are supported.");
					System.err.flush();
		            System.exit(-1);					
				}
				
				if (dupLines > 0 && holdBufferOffset > 15)
				{
					trcCtl.dsBufferOffset = holdBufferOffset;
					if (trcCtl.dsBufferOffset + TrcParser.dumpLineWidth * dupLines <= trcCtl.dsBufferMax)
					{
						for (int j=1; j <= dupLines; j++)
						{
							for (int k=16; k > 0; k--)
							{
								trcCtl.dsBuffer[trcCtl.dsBufferOffset] = trcCtl.dsBuffer[holdBufferOffset-k];
								trcCtl.dsBufferOffset++;
								
								if (trcCtl.dataStrcName != null && trcCtl.dataStrcName.equals("Buffer:"))
								{
									if (TrcParser.messageParsingGiven || TrcParser.messageSearchGiven || trcCtl.msg2File || trcCtl.msg2FileGlobal)
										MessageBufferUtil.messageBufferAddByte(trcCtl);
								
									if (TrcParser.messageParsingGiven)
										MessageParsingUtil.messageParsing(trcCtl, TrcParser.messageParsingValue, TrcParser.endianness);
								}
							}
						}
					}
				}
			}
		}
		
		if (badHexLine)
		{
			trcCtl.resetDataStrc();
			return;
		}
		
		if (holdBufferOffset == trcCtl.dsBufferOffset)
			trcCtl.hexLine = false;
		else
			trcCtl.hexLine = true;
		
		DataStrcUtil.processDataStrcName(trcCtl);
		
	}
	
	public static boolean checkForDupLine(String line) {
		boolean result = false;
		
		int x = line.indexOf("lines suppressed, same as above");
		
		if (x == -1)
			x = line.indexOf("Duplicate Lines =");
		
		if (x > 5)
			result = true;
		
		return result;
	}
	

	public static boolean writeMQLONG(TrcCtl trcCtl, String padStr, String label, int offset) {
		//NOTE: This method will only allow an input label to be up to length 19.  It performs reductions
		//      on this field to this max if it exceeds the limits.
		String str1 = null;
		String str2 = null;
		int labelLen = 0;
		int j = 0;
		int field = 0;
        byte [] bytes = null;
        
		if (offset + 4 > trcCtl.dsBufferOffset || offset + 4 > trcCtl.dataStrcSize)
		{
			trcCtl.resetDataStrc();
			return false;
		}
		
		field = TrcParser.dsBuffer4BytesToInt(trcCtl, offset);
						
		labelLen = label.length();
		if (labelLen > 19)
		{
			label = label.substring(0, 18);
			labelLen = 19;
		}
		str2 = "   ";
		str2 = str2 + label;
		str2 = str2 + " (MQLONG)";
		j = 20;
		for (int i=1; i < (j - labelLen); i++)
			str2 = str2 + " ";
		str2 = str2 + ": ";
		TrcParser.writeOutputFile(trcCtl, padStr + str2 + field);
		bytes = Arrays.copyOfRange(trcCtl.dsBuffer, offset, offset+4);
		str1 = TrcParser.bytesToHex(bytes);
		TrcParser.writeOutputFile(trcCtl, padStr + "                                 x'" + str1 +"'");
		padStr = padStr + "      ";
		TrcParser.writeConstants(trcCtl, padStr, label, field, null);
		
		return true;
	}
	
	public static boolean writeMQCHAR(TrcCtl trcCtl, String padStr, String label, int length, int offset) {

		//NOTE: This method will only allow an input label to be up to length 16, and an input length to be up to 999.  It performs reductions
		//      on these fields to this max if they exceed these limits.
		String str1 = null;
		String str2 = null;
		String holdStr = null;
		int labelLen = 0;
		int j = 0;
		byte [] bytes = null;
		
		if (offset + length > trcCtl.dsBufferOffset || offset + length > trcCtl.dataStrcSize)
		{
			trcCtl.resetDataStrc();
			return false;
		}
		
		str1 = TrcParser.getStringFromDSBuffer(trcCtl, label, offset, length);
		holdStr = str1;
		labelLen = label.length();
		if (labelLen > 16)
		{
			label = label.substring(0, 15);
			labelLen = 16;
		}
		str2 = "   ";
		str2 = str2 + label;
		if (length > 999)
			length = 999;
		str2 = str2 + " (MQCHAR" + length + ")";
		if (length < 10)
			j = 19;
		if (length > 9 && length < 100)
			j = 18;
		if (length > 99 && length < 1000)
			j = 17;
		for (int i=1; i < (j - labelLen); i++)
			str2 = str2 + " ";
		str2 = str2 + ": '";
		TrcParser.writeOutputFile(trcCtl, padStr + str2 + str1 +"'");
		bytes = Arrays.copyOfRange(trcCtl.dsBuffer, offset, offset+length);
		str1 = TrcParser.bytesToHex(bytes);
		TrcParser.writeOutputFile(trcCtl, padStr + "                                 x'" + str1 +"'");
		padStr = padStr + "      ";
		TrcParser.writeConstants(trcCtl, padStr, label, 0, holdStr);
		
		return true;
	}
	
	public static boolean writeMQCHARV(TrcCtl trcCtl, String padStr, String label, int offset) {

		//NOTE: This method will only allow an input label to be up to length 18.  It performs reductions
		//      on this field to this max if it exceeds this limit.
		String str2 = null;
		int labelLen = 0;
		int j = 0;
		
		if (offset + TrcParser.pointerLength + 16 > trcCtl.dsBufferOffset || offset + TrcParser.pointerLength + 16 > trcCtl.dataStrcSize) 
		{
			trcCtl.resetDataStrc();
			return false;
		}
		
		labelLen = label.length();
		if (labelLen > 18)
		{
			label = label.substring(0, 17);
			labelLen = 18;
		}
		str2 = "   ";
		str2 = str2 + label;
		str2 = str2 + " (MQCHARV)";
		j = 19;
		for (int i=1; i < (j - labelLen); i++)
			str2 = str2 + " ";
		str2 = str2 + ":";
		
		// MQCHARV
		TrcParser.writeOutputFile(trcCtl, padStr + str2);
		
		padStr = padStr + "   ";
		
		// VSPtr
		TrcParser.writeMQPTR(trcCtl, padStr, "VSPtr", TrcParser.pointerLength, offset);
		offset=offset+TrcParser.pointerLength;
		
		TrcParser.writeMQLONG(trcCtl, padStr, "VSOffset", offset);
		offset=offset+4;
		
		TrcParser.writeMQLONG(trcCtl, padStr, "VSBufSize", offset);
		offset=offset+4;
		
		TrcParser.writeMQLONG(trcCtl, padStr, "VSLength", offset);
		offset=offset+4;
		
		TrcParser.writeMQLONG(trcCtl, padStr, "VSCCSID", offset);
				
		return true;
	}
	
	public static boolean writeMQBYTE(TrcCtl trcCtl, String padStr, String label, int length, int offset) {

		//NOTE: This method will only allow an input label to be up to length 16, and an input length to be up to 999.  It performs reductions
		//      on these fields to this max if they exceed these limits.
		String str1 = null;
		String str2 = null;
		int labelLen = 0;
		int j = 0;
		byte [] bytes = null;
		
		if (offset + length > trcCtl.dsBufferOffset || offset + length > trcCtl.dataStrcSize)
		{
			trcCtl.resetDataStrc();
			return false;
		}
		
		labelLen = label.length();
		if (labelLen > 16)
		{
			label = label.substring(0, 15);
			labelLen = 16;
		}
		str2 = "   ";
		str2 = str2 + label;
		if (length > 999)
			length = 999;
		str2 = str2 + " (MQBYTE" + length + ")";
		if (length < 10)
			j = 19;
		if (length > 9 && length < 100)
			j = 18;
		if (length > 99 && length < 1000)
			j = 17;
		for (int i=1; i < (j - labelLen); i++)
			str2 = str2 + " ";
		str2 = str2 + ": x'";
		bytes = Arrays.copyOfRange(trcCtl.dsBuffer, offset, offset+length);
		str1 = TrcParser.bytesToHex(bytes);
		TrcParser.writeOutputFile(trcCtl, padStr + str2 + str1 +"'");
		
		return true;
	}
	
	public static boolean writeMQPTR(TrcCtl trcCtl, String padStr, String label, int length, int offset) {

		//NOTE: This method will only allow an input label to be up to length 18, and an input length to be up to 8.  It performs reductions
		//      on the input label field to this max if they exceeds its limit.
		String str1 = null;
		String str2 = null;
		int labelLen = 0;
		int j = 0;
		byte [] bytes = null;
		
		if (offset + length > trcCtl.dsBufferOffset || offset + length > trcCtl.dataStrcSize)
		{
			trcCtl.resetDataStrc();
			return false;
		}
		
		labelLen = label.length();
		if (labelLen > 18)
		{
			label = label.substring(0, 17);
			labelLen = 18;
		}
		str2 = "   ";
		str2 = str2 + label;
		if (length > 8)
			length = 8;
		str2 = str2 + " (MQPTR)";
		j = 21;
		for (int i=1; i < (j - labelLen); i++)
			str2 = str2 + " ";
		str2 = str2 + ": x'";
		bytes = Arrays.copyOfRange(trcCtl.dsBuffer, offset, offset+length);
		str1 = TrcParser.bytesToHex(bytes);
		TrcParser.writeOutputFile(trcCtl, padStr + str2 + str1 +"'");
		
		return true;
	}
	
	public static boolean writeEqualBytes(TrcCtl trcCtl, String padStr, String label, int length, int offset) {

		//NOTE: This method will only allow an input label to be up to length 16, and an input length to be up to 999.  It performs reductions
		//      on these fields to this max if they exceed these limits.
		String str1 = null;
		int labelLen = 0;
		byte [] bytes = null;
		
		if (offset + length > trcCtl.dsBufferOffset)
		{
			trcCtl.resetDataStrc();
			return false;
		}
		
		labelLen = label.length();
		if (labelLen > 16)
		{
			label = label.substring(0, 15);
			labelLen = 16;
		}
		
		bytes = Arrays.copyOfRange(trcCtl.dsBuffer, offset, offset+length);
		str1 = TrcParser.bytesToHex(bytes);
		TrcParser.writeOutputFile(trcCtl, padStr + "   " + label + "=" + str1);
		
		return true;
	}
	
	public static boolean writeEqualInt(TrcCtl trcCtl, String padStr, String label, int offset) {

		//NOTE: This method will only allow an input label to be up to length 16, and an input length to be up to 999.  It performs reductions
		//      on these fields to this max if they exceed these limits.
		int labelLen = 0;
		int field = 0;
		
		if (offset + 4 > trcCtl.dsBufferOffset)
		{
			trcCtl.resetDataStrc();
			return false;
		}
		
		labelLen = label.length();
		if (labelLen > 16)
		{
			label = label.substring(0, 15);
			labelLen = 16;
		}
		
		field = TrcParser.dsBuffer4BytesToInt(trcCtl, offset);
		TrcParser.writeOutputFile(trcCtl, padStr + "   " + label + "=" + field);
		
		return true;
	}
	
	public static void writeOutputFile(TrcCtl trcCtl, String line) {

		// For special case of resolving an input file/value, write to stdout and return
		if (TrcParser.constFieldGiven)
		{
			System.out.println(line);
			return;
		}
		
		try {
			byte b[] = (line + System.lineSeparator()).getBytes(TrcParser.inputFileCharset);
			trcCtl.outputTrcBufferedFileStream.write(b, 0, b.length);
		} catch (Exception ex) {
			System.err.println("Error encountered when writing to output file " + trcCtl.outputTrcFile);
			System.err.println("For line:");
			System.err.println(line);
        	ex.printStackTrace();
            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
              System.err.println("... Caused by ");
              t.printStackTrace();
            }
            System.err.flush();
            System.exit(-1);
		}
	}
	
	public static void writeOutputBufferedFileStream(BufferedOutputStream outputTrcBufferedFileStream, String outputTrcFile, String inputFileCharset, String line) {

		try {
			byte b[] = (line + System.lineSeparator()).getBytes(inputFileCharset);
			outputTrcBufferedFileStream.write(b, 0, b.length);
		} catch (Exception ex) {
			System.err.println("Error encountered when writing to output file " + outputTrcFile);
			System.err.println("For line:");
			System.err.println(line);
        	ex.printStackTrace();
            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
              System.err.println("... Caused by ");
              t.printStackTrace();
            }
            System.err.flush();
            System.exit(-1);
		}
	}

	
	public static int dsBuffer4BytesToInt (TrcCtl trcCtl, int offset) {
		byte [] bytes = new byte[4];
		if (TrcParser.endianness.equals("little"))
		{
			bytes[3] = trcCtl.dsBuffer[offset];
			bytes[2] = trcCtl.dsBuffer[offset+1];
			bytes[1] = trcCtl.dsBuffer[offset+2];
			bytes[0] = trcCtl.dsBuffer[offset+3];
		}
		else
		{
			bytes[0] = trcCtl.dsBuffer[offset];
			bytes[1] = trcCtl.dsBuffer[offset+1];
			bytes[2] = trcCtl.dsBuffer[offset+2];
			bytes[3] = trcCtl.dsBuffer[offset+3];
		}
		
		return ByteBuffer.wrap(bytes).getInt();
	}
	
	public static String bytesToHex(byte[] bytes) {
	    char[] hexChars = new char[bytes.length * 2];
	    for ( int j = 0; j < bytes.length; j++ ) {
	        int v = bytes[j] & 0xFF;
	        if (TrcParser.traceType.equals("windows"))
	        {
	        	hexChars[j * 2] = hexArrayWin[v >>> 4];
	        	hexChars[j * 2 + 1] = hexArrayWin[v & 0x0F];
	        }
	        else
	        {
	        	hexChars[j * 2] = hexArray[v >>> 4];
	        	hexChars[j * 2 + 1] = hexArray[v & 0x0F];
	        }
	    }
	    return new String(hexChars);
	}
	
	public static String getStringFromDSBuffer (TrcCtl trcCtl, String label, int offset, int length) {
		
		String str1 = "";
		boolean adjustByte = false;
				
		for (int i = 0; i < length; i++)
		{
			adjustByte = false;
			
			if (TrcParser.printCharStr.equals("Linux_x86"))
			{
				//Adjust certain bytes to match Linux_x86 based strmqtrc printable formatting 
				if (trcCtl.dsBuffer[offset+i] <= 8 || (trcCtl.dsBuffer[offset+i] >= 14 && trcCtl.dsBuffer[offset+i] <= 31) || trcCtl.dsBuffer[offset+i] == 127)
				{
					str1 += new String(".");
					adjustByte = true;
				}
				
				if (trcCtl.dsBuffer[offset+i] >= 9 && trcCtl.dsBuffer[offset+i] <= 13)
				{
					str1 += new String(" ");
					adjustByte = true;
				}
			}
			
			if (TrcParser.printCharStr.equals("ASCII"))
			{
				//Adjust certain bytes to match ASCII based printable formatting 
				if (trcCtl.dsBuffer[offset+i] <= 31 || trcCtl.dsBuffer[offset+i] == 127)
				{
					str1 += new String(".");
					adjustByte = true;
				}

			}
			
			if (!adjustByte)
			{
				try {
					str1 += new String(trcCtl.dsBuffer, offset+i, 1, TrcParser.inputFileCharset);
				} catch (Exception ex) {
					System.err.println("Error encountered when processing MQCHAR for field " + label + " and output file " + trcCtl.outputTrcFile);
		        	ex.printStackTrace();
		            for (Throwable t = ex.getCause(); t != null; t = t.getCause()) {
		              System.err.println("... Caused by ");
		              t.printStackTrace();
		            }
		            System.err.flush();
		            System.exit(-1);
				}		
			}
		}
		
		return str1;
		
	}
	
	public static String padString (TrcCtl trcCtl) {
		String padStr = "";
				
		if (trcCtl.pidTid.equals("NOTFOUND"))
			return padStr;
		
		for (int i = 0; i < trcCtl.pidTidPos; i++)
			padStr = padStr + " ";
		
		padStr = padStr + trcCtl.pidTid;
		
		int j = trcCtl.indent - trcCtl.pidTidPos - trcCtl.pidTid.length();
		
		if (j < 2)
		{
			padStr = "";
			return padStr;
		}
		
		for (int i = 0; i < j; i++)
			padStr = padStr + " ";
		
		// provide extra padding for windows trace
		if (TrcParser.traceType.equals("windows"))
			padStr = padStr + "      ";
		
		return padStr;
		
	}
	
	public static void listConstFieldValidValues() {
		System.out.println("Valid constantField values are the following:");
		System.out.println("Action, CallBackType, CallType, CCSID, ValueCCSID, RequestedCCSID, ReturnedCCSID, CodedCharSetId, CompCode, CopyOptions, Encoding, RequestedEncoding, ReturnedEncoding, ValueEncoding, Expiry, Feedback, Flags, GroupStatus, MatchOptions, MsgFlags, MsgType, MQCBDO.Options, MQCMHO.Options, MQCNO.Options, MQCO.Options, MQCTLO.Options, MQDMHO.Options, MQGMO.Options, MQIMPO.Options, MQOO.Options, OpenOptions, MQPD.Options, MQPMO.Options, MQSD.Options, SubOptions, MQSMPO.Options, ObjectType, OriginalLength, PD.Context, Persistence, Priority, PubPriority, PutApplType, PutMsgRecFields, ReconnectDelay, Report, ResolvedType, Segmentation, SegmentStatus, Signal1, StatType, State, SubExpiry, Support, Type, WaitInterval");
	}
	
	public static void writeConstants(TrcCtl trcCtl, String padStr, String label, int field, String str1) {
		switch (label) {
		case "Action" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Action MQACTP_NEW"); 
               break;
            case 1: 
               TrcParser.writeOutputFile(trcCtl, padStr + "Action MQACTP_FORWARD");
               break;
            case 2: 
               TrcParser.writeOutputFile(trcCtl, padStr + "Action MQACTP_REPLY");
               break;
            case 3: 
               TrcParser.writeOutputFile(trcCtl, padStr + "Action MQACTP_REPORT");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Action UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "CallBackType" :
            switch (field) {
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallBackType MQCBT_MESSAGE_CONSUMER");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallBackType MQCBT_EVENT_HANDLER");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallBackType UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "CallType" :
            switch (field) {
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType MQCBCT_START_CALL");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType MQCBCT_STOP_CALL");
               break;
            case 3:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType MQCBCT_REGISTER_CALL");
               break;
            case 4:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType MQCBCT_DEREGISTER_CALL");
               break;
            case 5:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType MQCBCT_EVENT_CALL");
               break;
            case 6:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType MQCBCT_MSG_REMOVED");
               break;
            case 7:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType MQCBCT_MSG_NOT_REMOVED");
               break;
            case 8:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType MQCBCT_MC_EVENT_CALL");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "CallType UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "CCSID" :
		case "ValueCCSID" :
		case "RequestedCCSID" :
		case "ReturnedCCSID" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "CCSID MQCCSI_UNDEFINED");
               TrcParser.writeOutputFile(trcCtl, padStr + "CCSID MQCCSI_DEFAULT");
               TrcParser.writeOutputFile(trcCtl, padStr + "CCSID MQCCSI_Q_MGR");
               break;
            case -1:
               TrcParser.writeOutputFile(trcCtl, padStr + "CCSID MQCCSI_EMBEDDED");
               break;
            case -2:
               TrcParser.writeOutputFile(trcCtl, padStr + "CCSID MQCCSI_INHERIT");
               break;
            case -3:
               TrcParser.writeOutputFile(trcCtl, padStr + "CCSID MQCCSI_APPL");
               break;
            case -4:
                TrcParser.writeOutputFile(trcCtl, padStr + "CCSID MQCCSI_AS_PUBLISHED");
                break;
            }
			break;
		case "CodedCharSetId" :
			switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "CodedCharSetId MQCCSI_UNDEFINED");
               TrcParser.writeOutputFile(trcCtl, padStr + "CodedCharSetId MQCCSI_DEFAULT");
               TrcParser.writeOutputFile(trcCtl, padStr + "CodedCharSetId MQCCSI_Q_MGR");
               break;
            case -1:
               TrcParser.writeOutputFile(trcCtl, padStr + "CodedCharSetId MQCCSI_EMBEDDED");
               break;
            case -2:
               TrcParser.writeOutputFile(trcCtl, padStr + "CodedCharSetId MQCCSI_INHERIT");
               break;
            case -3:
               TrcParser.writeOutputFile(trcCtl, padStr + "CodedCharSetId MQCCSI_APPL");
               break;
            case -4:
               TrcParser.writeOutputFile(trcCtl, padStr + "CodedCharSetId MQCCSI_AS_PUBLISHED");
               break;
            }    
			break;
		case "CompCode" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "CompCode MQCC_OK");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "CompCode MQCC_WARNING");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "CompCode MQCC_FAILED");
               break;
            case -1:
               TrcParser.writeOutputFile(trcCtl, padStr + "CompCode MQCC_UNKNOWN");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "CompCode UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "CopyOptions" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "CopyOptions MQCOPY_NONE");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "CopyOptions MQCOPY_ALL");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "CopyOptions MQCOPY_FORWARD");
               break;
            case 4:
               TrcParser.writeOutputFile(trcCtl, padStr + "CopyOptions MQCOPY_PUBLISH");
               break;
            case 8:
               TrcParser.writeOutputFile(trcCtl, padStr + "CopyOptions MQCOPY_REPLY");
               break;
            case 16:
               TrcParser.writeOutputFile(trcCtl, padStr + "CopyOptions MQCOPY_REPORT");
               break;
            case 22:
               TrcParser.writeOutputFile(trcCtl, padStr + "CopyOptions MQCOPY_DEFAULT");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "CopyOptions UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "Encoding" :
		case "RequestedEncoding" :
		case "ReturnedEncoding" :
		case "ValueEncoding" :
            switch (field  & 0x0000000F) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_INTEGER_UNDEFINED");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_INTEGER_NORMAL");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_INTEGER_REVERSED");
               break;
            }

            switch (field & 0x000000F0) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_DECIMAL_UNDEFINED");
               break;
            case 16:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_DECIMAL_NORMAL");
               break;
            case 32:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_DECIMAL_REVERSED");
               break;
            }

            switch (field & 0x00000F00) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_FLOAT_UNDEFINED");
               break;
            case 256:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_FLOAT_IEEE_NORMAL");
               break;
            case 512:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_FLOAT_IEEE_REVERSED");
               break;
            case 768:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_FLOAT_S390");
               break;
            case 1024:
               TrcParser.writeOutputFile(trcCtl, padStr + "Encoding MQENC_FLOAT_TNS");
               break;
            }
			break;
		case "Expiry" :
			if (field == -1)
				TrcParser.writeOutputFile(trcCtl, padStr + "Expiry MQEI_UNLIMITTED");
			break;
		case "Feedback" :
			switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_NONE");
               break;
            case 256:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_QUIT");
               break;
            case 258:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_EXPIRATION");
               break;
            case 259:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_COA");
               break;
            case 260:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_COD");
               break;
            case 262:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CHANNEL_COMPLETED");
               break;
            case 263:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CHANNEL_FAIL_RETRY");
               break;
            case 264:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CHANNEL_FAIL");
               break;
            case 265:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_APPL_CANNOT_BE_STARTED");
               break;
            case 266:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_TM_ERROR");
               break;
            case 267:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_APPL_TYPE_ERROR");
               break;
            case 268:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_STOPPED_BY_MSG_EXIT");
               break;
            case 269:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_ACTIVITY");
               break;
            case 271:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_XMIT_Q_MSG_ERROR");
               break;
            case 275:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_PAN");
               break;
            case 276:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_NAN");
               break;
            case 277:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_STOPPED_BY_CHAD_EXIT");
               break;
            case 279:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_STOPPED_BY_PUBSUB_EXIT");
               break;
            case 280:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_NOT_A_REPOSITORY_MSG");
               break;
            case 281:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_BIND_OPEN_CLUSRCVR_DEL");
               break;
            case 282:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_MAX_ACTIVITIES");
               break;
            case 283:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_NOT_FORWARDED");
               break;
            case 284:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_NOT_DELIVERED");
               break;
            case 285:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_UNSUPPORTED_FORWARDING");
               break;
            case 286:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_UNSUPPORTED_DELIVERY");
               break;
            case 291:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_DATA_LENGTH_ZERO");
               break;
            case 292:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_DATA_LENGTH_NEGATIVE");
               break;
            case 293:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_DATA_LENGTH_TOO_BIG");
               break;
            case 294:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_BUFFER_OVERFLOW");
               break;
            case 295:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_LENGTH_OFF_BY_ONE");
               break;
            case 296:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_IIH_ERROR");
               break;
            case 298:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_NOT_AUTHORIZED_FOR_IMS");
               break;
            case 300:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_IMS_ERROR");
               break;
            case 301:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_IMS_FIRST");
               break;
            case 399:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_IMS_LAST");
               break;
            case 401:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_INTERNAL_ERROR");
               break;
            case 402:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_NOT_AUTHORIZED");
               break;
            case 403:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_BRIDGE_FAILURE");
               break;
            case 404:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_CORREL_ID_ERROR");
               break;
            case 405:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_CCSID_ERROR");
               break;
            case 406:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_ENCODING_ERROR");
               break;
            case 407:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_CIH_ERROR");
               break;
            case 408:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_UOW_ERROR");
               break;
            case 409:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_COMMAREA_ERROR");
               break;
            case 410:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_APPL_NOT_STARTED");
               break;
            case 411:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_APPL_ABENDED");
               break;
            case 412:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_DLQ_ERROR");
               break;
            case 413:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_CICS_UOW_BACKED_OUT");
               break;
            case 501:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_PUBLICATIONS_ON_REQUEST");
               break;
            case 502:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_SUBSCRIBER_IS_PUBLISHER");
               break;
            case 503:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_MSG_SCOPE_MISMATCH");
               break;
            case 504:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_SELECTOR_MISMATCH");
               break;
            case 505:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_NOT_A_GROUPUR_MSG");
               break;
            case 600:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_IMS_NACK_1A_REASON_FIRST");
               break;
            case 855:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQFB_IMS_NACK_1A_REASON_LAST");
               break;
            case 2030:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQRC_MSG_TOO_BIG_FOR_Q");
               break;
            case 2031:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQRC_MSG_TOO_BIG_FOR_Q_MGR");
               break;
            case 2035:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQRC_NOT_AUTHORIZED");
               break;
            case 2048:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQRC_PERSISTENT_NOT_ALLOWED");
               break;
            case 2051:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQRC_PUT_INHIBITED");
               break;
            case 2053:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQRC_Q_FULL");
               break;
            case 2056:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback MQRC_Q_SPACE_NOT_AVAILABLE");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Feedback UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "Flags" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Flags MQCBCF_NONE");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Flags MQCBCF_READA_BUFFER_EMPTY");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Flags UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "GroupStatus" :
            switch (str1) {
            case " ":
               TrcParser.writeOutputFile(trcCtl, padStr + "GroupStatus MQGS_NOT_IN_GROUP");
               break;
            case "G":
               TrcParser.writeOutputFile(trcCtl, padStr + "GroupStatus MQGS_MSG_IN_GROUP");
               break;
            case "L":
               TrcParser.writeOutputFile(trcCtl, padStr + "GroupStatus MQGS_LAST_MSG_IN_GROUP");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "GroupStatus UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "MatchOptions" :
            if (field == 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "MatchOptions=MQMO_NONE");
             if ((field & 0x1) == 0x1)
                TrcParser.writeOutputFile(trcCtl, padStr + "MatchOptions=MQMO_MATCH_MSG_ID");
             if ((field & 0x2) == 0x2)
                TrcParser.writeOutputFile(trcCtl, padStr + "MatchOptions=MQMO_MATCH_CORREL_ID");
             if ((field & 0x4) == 0x4)
                TrcParser.writeOutputFile(trcCtl, padStr + "MatchOptions=MQMO_MATCH_GROUP_ID");
             if ((field & 0x8) == 0x8)
                TrcParser.writeOutputFile(trcCtl, padStr + "MatchOptions=MQMO_MATCH_MSG_SEQ_NUMBER");
             if ((field & 0x10) == 0x10)
                TrcParser.writeOutputFile(trcCtl, padStr + "MatchOptions=MQMO_MATCH_OFFSET");
             if ((field & 0x20) == 0x20)
                TrcParser.writeOutputFile(trcCtl, padStr + "MatchOptions=MQMO_MATCH_MSG_TOKEN");
             if ((field & 0xFFFFFFC0) != 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "MatchOptions=UNKNOWN_CONSTANT in range 0xFFFFFFC0");
			break;
		case "MsgFlags" :
			if (field == 0)
            {
				TrcParser.writeOutputFile(trcCtl, padStr + "MsgFlags MQMF_NONE");
				TrcParser.writeOutputFile(trcCtl, padStr + "MsgFlags MQMF_SEGMENTATION_INHIBITED");
            }
			if ((field & 0x1) == 0x1)
				TrcParser.writeOutputFile(trcCtl, padStr + "MsgFlags MQMF_SEGMENTATION_ALLOWED");
			if ((field & 0x2) == 0x2)
				TrcParser.writeOutputFile(trcCtl, padStr + "MsgFlags MQMF_SEGMENT");
			if ((field & 0x4) == 0x4)
				TrcParser.writeOutputFile(trcCtl, padStr + "MsgFlags MQMF_LAST_SEGMENT");
			if ((field & 0x8) == 0x8)
				TrcParser.writeOutputFile(trcCtl, padStr + "MsgFlags MQMF_MSG_IN_GROUP");
			if ((field & 0x10) == 0x10)
				TrcParser.writeOutputFile(trcCtl, padStr + "MsgFlags MQMF_LAST_MSG_IN_GROUP");
			if ((field & 0xFFFFFFE0) != 0)
				TrcParser.writeOutputFile(trcCtl, padStr + "MsgFlags UNKNOWN_CONSTANT in range 0xFFFFFFE0");
			break;
		case "MsgType" :
			switch (field) {
            case 1:
            	TrcParser.writeOutputFile(trcCtl, padStr + "MsgType MQMT_REQUEST");
                break;
            case 2:
            	TrcParser.writeOutputFile(trcCtl, padStr + "MsgType MQMT_REPLY");
                break;
            case 4:
            	TrcParser.writeOutputFile(trcCtl, padStr + "MsgType MQMT_REPORT");
                break;
            case 8:
            	TrcParser.writeOutputFile(trcCtl, padStr + "MsgType MQMT_DATAGRAM");
                break;
            case 112:
               	TrcParser.writeOutputFile(trcCtl, padStr + "MsgType MQMT_MQE_FIELDS_FROM_MQE");
                break;
            case 113:
               	TrcParser.writeOutputFile(trcCtl, padStr + "MsgType MQMT_MQE_FIELDS");
                break;
            default:
              	TrcParser.writeOutputFile(trcCtl, padStr + "MsgType UNKNOWN_CONSTANT");
                break;
            }
			break;
		case "MQCBDO.Options" :
            if (field == 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO_NONE");

             if ((field & 0x1) == 0x1)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO_START_CALL");

             if ((field & 0x2) == 0x2)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO UNKNOWN_CONSTANT 0x2");

             if ((field & 0x4) == 0x4)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO_STOP_CALL");

             if ((field & 0x8) == 0x8)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO UNKNOWN_CONSTANT 0x8");

             if ((field & 0x000000F0) != 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO UNKNOWN_CONSTANT in range 0x000000F0");

             if ((field & 0x100) == 0x100)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO_REGISTER_CALL");

             if ((field & 0x200) == 0x200)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO_DEREGISTER_CALL");

             if ((field & 0x400) == 0x400)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO UNKNOWN_CONSTANT 0x400");

             if ((field & 0x800) == 0x800)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO UNKNOWN_CONSTANT 0x800");

             if ((field & 0x1000) == 0x1000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO UNKNOWN_CONSTANT 0x1000");

             if ((field & 0x2000) == 0x2000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO_FAIL_IF_QUIESCING");

             if ((field & 0x4000) == 0x4000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO_EVENT_CALL");

             if ((field & 0x8000) == 0x8000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO_MC_EVENT_CALL");

             if ((field & 0xFFFF0000) != 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCBDO UNKNOWN_CONSTANT in range 0xFFFF0000");
			break;
		case "MQCMHO.Options" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCMHO_DEFAULT_VALIDATION");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCMHO_NO_VALIDATION");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCMHO_VALIDATE");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCMHO UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "MQCNO.Options" :
			if (field == 0)
				TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_NONE");
			else
            {
             if ((field & 0x1) == 0x1)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_FASTPATH_BINDING");

             if ((field & 0x2) == 0x2)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_SERIALIZE_CONN_TAG_Q_MGR");

             if ((field & 0x4) == 0x4)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_SERIALIZE_CONN_TAG_QSG");

             if ((field & 0x8) == 0x8)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_RESTRICT_CONN_TAG_Q_MGR");

             if ((field & 0x10) == 0x10) 
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_RESTRICT_CONN_TAG_QSG");

             if ((field & 0x20) == 0x20) 
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_HANDLE_SHARE_NONE");

             if ((field & 0x40) == 0x40) 
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_HANDLE_SHARE_BLOCK");

             if ((field & 0x80) == 0x80) 
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_HANDLE_SHARE_NO_BLOCK");

             if ((field & 0x100) == 0x100) 
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_SHARED_BINDING");

             if ((field & 0x200) == 0x200)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_ISOLATED_BINDING");

             if ((field & 0x400) == 0x400)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_LOCAL_BINDING");

             if ((field & 0x800) == 0x800)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_CLIENT_BINDING");

             if ((field & 0x1000) == 0x1000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_ACCOUNTING_MQI_ENABLED");

             if ((field & 0x2000) == 0x2000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_ACCOUNTING_MQI_DISABLED");

             if ((field & 0x4000) == 0x4000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_ACCOUNTING_Q_ENABLED");

             if ((field & 0x8000) == 0x8000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_ACCOUNTING_Q_DISABLED");

             if ((field & 0x10000) == 0x10000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_NO_CONV_SHARING");

             if ((field & 0x20000) == 0x20000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO UNKNOWN_CONSTANT 0x20000");

             if ((field & 0x40000) == 0x40000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_ALL_CONVS_SHARE");

             if ((field & 0x80000) == 0x80000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_CD_FOR_OUTPUT_ONLY");

             if ((field & 0x100000) == 0x100000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_USE_CD_SELECTION");

             if ((field & 0x200000) == 0x200000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_GENERATE_CONN_TAG");

             if ((field & 0x400000) == 0x400000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO UNKNOWN_CONSTANT 0x400000");

             if ((field & 0x800000) == 0x800000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO UNKNOWN_CONSTANT 0x800000");

             if ((field & 0x1000000) == 0x1000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_RECONNECT");

             if ((field & 0x2000000) == 0x2000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_RECONNECT_DISABLED");

             if ((field & 0x4000000) == 0x4000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_RECONNECT_Q_MGR");

             if ((field & 0x8000000) == 0x8000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_ACTIVITY_TRACE_ENABLED");

             if ((field & 0x10000000) == 0x10000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO_ACTIVITY_TRACE_DISABLED");

             if ((field & 0x20000000) == 0x20000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO UNKNOWN_CONSTANT 0x20000000");

             if ((field & 0x40000000) == 0x40000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO UNKNOWN_CONSTANT 0x40000000");

             if ((field & 0x80000000) == 0x80000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCNO UNKNOWN_CONSTANT 0x80000000");

            }
			break;
		case "MQCO.Options" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCO_NONE");
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCO_IMMEDIATE");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCO_DELETE");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCO_DELETE_PURGE");
               break;
            case 4:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCO_KEEP_SUB");
               break;
            case 8:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCO_REMOVE_SUB");
               break;
            case 32:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCO_QUIESCE");
               break;
            default: 
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCO UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "MQCTLO.Options" :
            if (field == 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCTLO_NONE");
             else
             {
                 if ((field & 0x1) == 0x1)
                    TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCTLO_THREAD_AFFINITY");

                 if ((field & 0x2000) == 0x2000)
                    TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCTLO_FAIL_IF_QUIESCING");

                 if ((field & 0xFFFFD00E) != 0)
                    TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQCTLO UNKNOWN_CONSTANT in range 0xFFFFD00E");
             }
			break;
		case "MQDMHO.Options" :
            switch (field) {
            case 0:
            	TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQDMHO_NONE");
               break;
            default:
            	TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQDMHO UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "MQGMO.Options" :
            if (field == 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_NONE");

             if ((field & 0x1) == 0x1)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_WAIT");

             if ((field & 0x2) == 0x2)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_SYNCPOINT");

             if ((field & 0x4) == 0x4)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_NO_SYNCPOINT");

             if ((field & 0x8) == 0x8)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_SET_SIGNAL");

             if ((field & 0x10) == 0x10)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_BROWSE_FIRST");

             if ((field & 0x20) == 0x20)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_BROWSE_NEXT");

             if ((field & 0x40) == 0x40)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_ACCEPT_TRUNCATED_MSG");

             if ((field & 0x80) == 0x80)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_MARK_SKIP_BACKOUT");

             if ((field & 0x100) == 0x100)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_MSG_UNDER_CURSOR");

             if ((field & 0x200) == 0x200)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_LOCK");

             if ((field & 0x400) == 0x400)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_UNLOCK");

             if ((field & 0x800) == 0x800)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_BROWSE_MSG_UNDER_CURSOR");

             if ((field & 0x1000) == 0x1000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_SYNCPOINT_IF_PERSISTENT");

             if ((field & 0x2000) == 0x2000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_FAIL_IF_QUIESCING");

             if ((field & 0x4000) == 0x4000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_CONVERT");

             if ((field & 0x8000) == 0x8000) 
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_LOGICAL_ORDER");

             if ((field & 0x10000) == 0x10000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_COMPLETE_MSG");

             if ((field & 0x20000) == 0x20000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_ALL_MSGS_AVAILABLE");

             if ((field & 0x40000) == 0x40000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_ALL_SEGMENTS_AVAILABLE");

             if ((field & 0x80000) == 0x80000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO UNKNOWN_CONSTANT 0x80000");

             if ((field & 0x100000) == 0x100000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_MARK_BROWSE_HANDLE");

             if ((field & 0x200000) == 0x200000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_MARK_BROWSE_CO_OP");

             if ((field & 0x400000) == 0x400000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_UNMARK_BROWSE_CO_OP");

             if ((field & 0x800000) == 0x800000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_UNMARK_BROWSE_HANDLE");

             if ((field & 0x1000000) == 0x1000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_UNMARKED_BROWSE_MSG");

             if ((field & 0x2000000) == 0x2000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_PROPERTIES_FORCE_MQRFH2");

             if ((field & 0x4000000) == 0x4000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_NO_PROPERTIES");

             if ((field & 0x8000000) == 0x8000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_PROPERTIES_IN_HANDLE");

             if ((field & 0x10000000) == 0x10000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_PROPERTIES_COMPATIBILITY");

             if ((field & 0x20000000) == 0x20000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO UNKNOWN_CONSTANT 0x20000000");

             if ((field & 0x40000000) == 0x40000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO UNKNOWN_CONSTANT 0x40000000");

             if ((field & 0x80000000) == 0x80000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO UNKNOWN_CONSTANT 0x80000000");

             if ((field & 0x01100010) == 0x01100010)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_BROWSE_HANDLE");

             if ((field & 0x01200010) == 0x01200010)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQGMO_BROWSE_CO_OP");
			break;
		case "MQIMPO.Options" :
            if (field == 0)
            {
             TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO_NONE");
             TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO_INQ_FIRST");
            }
	         if ((field & 0x1) == 0x1)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO UNKNOWN_CONSTANT 0x1");
	
	         if ((field & 0x2) == 0x2)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO_CONVERT_TYPE");
	
	         if ((field & 0x4) == 0x4)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO_QUERY_LENGTH");
	
	         if ((field & 0x8) == 0x8)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO_INQ_NEXT");
	
	         if ((field & 0x10) == 0x10)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO_INQ_PROP_UNDER_CURSOR");
	
	         if ((field & 0x20) == 0x20)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO_CONVERT_VALUE");
	
	         if ((field & 0xFFFFFFD0) != 0)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQIMPO UNKNOWN_CONSTANT in range 0xFFFFFFD0");
	         
			break;
		case "MQOO.Options" :
		case "OpenOptions" :
            if ((field & 0x1) == 0x1)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_INPUT_AS_Q_DEF");

             if ((field & 0x2) == 0x2)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_INPUT_SHARED");

             if ((field & 0x4) == 0x4)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_INPUT_EXCLUSIVE");

             if ((field & 0x8) == 0x8)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_BROWSE");

             if ((field & 0x10) == 0x10)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_OUTPUT");

             if ((field & 0x20) == 0x20)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_INQUIRE");

             if ((field & 0x40) == 0x40)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_SET");

             if ((field & 0x80) == 0x80)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_SAVE_ALL_CONTEXT");

             if ((field & 0x100) == 0x100)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_PASS_IDENTITY_CONTEXT");

             if ((field & 0x200) == 0x200)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_PASS_ALL_CONTEXT");

             if ((field & 0x400) == 0x400)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_SET_IDENTITY_CONTEXT");

             if ((field & 0x800) == 0x800)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_SET_ALL_CONTEXT");

             if ((field & 0x1000) == 0x1000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_ALTERNATE_USER_AUTHORITY");

             if ((field & 0x2000) == 0x2000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_FAIL_IF_QUIESCING");

             if ((field & 0x4000) == 0x4000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_BIND_ON_OPEN");

             if ((field & 0x8000) == 0x8000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_BIND_NOT_FIXED");

             if ((field & 0x10000) == 0x10000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_RESOLVE_NAMES");

             if ((field & 0x20000) == 0x20000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_CO_OP");

             if ((field & 0x40000) == 0x40000)
                {
                 TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_RESOLVE_LOCAL_TOPIC");
                 TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_RESOLVE_LOCAL_Q (C++ only)");
                }

             if ((field & 0x80000) == 0x80000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_NO_READ_AHEAD");

             if ((field & 0x100000) == 0x100000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_READ_AHEAD");

             if ((field & 0x200000) == 0x200000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_NO_MULTICAST");

             if ((field & 0x400000) == 0x400000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO_BIND_ON_GROUP");

             if ((field & 0x800000) == 0x800000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x800000");

             if ((field & 0x1000000) == 0x1000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x1000000");

             if ((field & 0x2000000) == 0x2000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x2000000");

             if ((field & 0x4000000) == 0x4000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x4000000");

             if ((field & 0x8000000) == 0x8000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x8000000");

             if ((field & 0x10000000) == 0x10000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x10000000");
            
             if ((field & 0x20000000) == 0x20000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x20000000");

             if ((field & 0x40000000) == 0x40000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x40000000");
            
             if ((field & 0x80000000) == 0x80000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQOO UNKNOWN_CONSTANT 0x80000000");
			break;
		case "MQPD.Options" :
            switch (field) {
            case 0:
            	TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPD_NONE");
               break;
            default:
            	TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPD UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "MQPMO.Options" :
            if (field == 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_NONE");

             if ((field & 0x1) == 0x1)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x1");

             if ((field & 0x2) == 0x2)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_SYNCPOINT");

             if ((field & 0x4) == 0x4)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_NO_SYNCPOINT");

             if ((field & 0x8) == 0x8)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x8");

             if ((field & 0x10) == 0x10)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x10");

             if ((field & 0x20) == 0x20)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_DEFAULT_CONTEXT");

             if ((field & 0x40) == 0x40)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_NEW_MSG_ID");

             if ((field & 0x80) == 0x80)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_NEW_CORREL_ID");

             if ((field & 0x100) == 0x100)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_PASS_IDENTITY_CONTEXT");

             if ((field & 0x200) == 0x200)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_PASS_ALL_CONTEXT");

             if ((field & 0x400) == 0x400)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_SET_IDENTITY_CONTEXT");

             if ((field & 0x800) == 0x800)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_SET_ALL_CONTEXT");

             if ((field & 0x1000) == 0x1000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_ALTERNATE_USER_AUTHORITY");

             if ((field & 0x2000) == 0x2000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_FAIL_IF_QUIESCING");

             if ((field & 0x4000) == 0x4000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_NO_CONTEXT");

             if ((field & 0x8000) == 0x8000) 
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_LOGICAL_ORDER");

             if ((field & 0x10000) == 0x10000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_ASYNC_RESPONSE");

             if ((field & 0x20000) == 0x20000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_SYNC_RESPONSE");

             if ((field & 0x40000) == 0x40000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_RESOLVE_LOCAL_Q");

             if ((field & 0x80000) == 0x80000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_WARN_IF_NO_SUBS_MATCHED");

             if ((field & 0x100000) == 0x100000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x100000");

             if ((field & 0x200000) == 0x200000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_RETAIN");

             if ((field & 0x400000) == 0x400000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x400000");

             if ((field & 0x800000) == 0x800000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_MD_FOR_OUTPUT_ONLY");

             if ((field & 0x1000000) == 0x1000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x1000000");

             if ((field & 0x2000000) == 0x2000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x2000000");

             if ((field & 0x4000000) == 0x4000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_SCOPE_QMGR");

             if ((field & 0x8000000) == 0x8000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_SUPPRESS_REPLYTO");

             if ((field & 0x10000000) == 0x10000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO_NOT_OWN_SUBS");

             if ((field & 0x20000000) == 0x20000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x20000000");

             if ((field & 0x40000000) == 0x40000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x40000000");

             if ((field & 0x80000000) == 0x80000000)
                TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQPMO UNKNOWN_CONSTANT 0x80000000");
			break;
		case "MQSD.Options" :
		case "SubOptions" :
            if (field == 0)
            {
             TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_NONE");
             TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_NON_DURABLE");
             TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_READ_AHEAD_AS_Q_DEF"); 
            }
	         if ((field & 0x1) == 0x1)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_ALTER");
	
	         if ((field & 0x2) == 0x2)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_CREATE");
	
	         if ((field & 0x4) == 0x4)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_RESUME");
	
	         if ((field & 0x8) == 0x8)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_DURABLE");
	
	         if ((field & 0x10) == 0x10)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_GROUP_SUB");
	
	         if ((field & 0x20) == 0x20)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_MANAGED");
	
	         if ((field & 0x40) == 0x40)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_SET_IDENTITY_CONTEXT");
	
	         if ((field & 0x80) == 0x80)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_NO_MULTICAST");
	
	         if ((field & 0x100) == 0x100)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_FIXED_USERID");
	
	         if ((field & 0x200) == 0x200)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_ANY_USERID");
	
	         if ((field & 0x400) == 0x400)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x400");
	
	         if ((field & 0x800) == 0x800)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_PUBLICATIONS_ON_REQUEST");
	
	         if ((field & 0x1000) == 0x1000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_NEW_PUBLICATIONS_ONLY");
	
	         if ((field & 0x2000) == 0x2000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_FAIL_IF_QUIESCING");
	
	         if ((field & 0x4000) == 0x4000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x4000");
	
	         if ((field & 0x8000) == 0x8000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x8000");
	
	         if ((field & 0x10000) == 0x10000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x10000");
	
	         if ((field & 0x20000) == 0x20000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x20000");
	
	         if ((field & 0x40000) == 0x40000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_ALTERNATE_USER_AUTHORITY");
	
	         if ((field & 0x80000) == 0x80000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x80000");
	
	         if ((field & 0x100000) == 0x100000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_WILDCARD_CHAR");
	
	         if ((field & 0x200000) == 0x200000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_WILDCARD_TOPIC");
	
	         if ((field & 0x400000) == 0x400000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_SET_CORREL_ID");    
	
	         if ((field & 0x800000) == 0x800000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x800000");
	
	         if ((field & 0x1000000) == 0x1000000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x1000000");
	
	         if ((field & 0x2000000) == 0x2000000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT 0x2000000");
	
	         if ((field & 0x4000000) == 0x4000000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_SCOPE_QMGR"); 
	
	         if ((field & 0x8000000) == 0x8000000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_NO_READ_AHEAD"); 
	
	         if ((field & 0x10000000) == 0x10000000)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO_READ_AHEAD"); 
	
	         if ((field & 0xE0000000) != 0)
	            TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSO UNKNOWN_CONSTANT in range 0xE0000000");
	         break;
		case "MQSMPO.Options" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSMPO_SET_FIRST");
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSMPO_NONE");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSMPO_SET_PROP_UNDER_CURSOR");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSMPO_SET_PROP_AFTER_CURSOR");
               break;
            case 4:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSMPO_APPEND_PROPERTY");
               break;
            case 8:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSMPO_SET_PROP_BEFORE_CURSOR");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Options=MQSMPO UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "ObjectType" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_NONE");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_Q");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_NAMELIST");
               break;
            case 3:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_PROCESS");
               break;
            case 4:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_STORAGE_CLASS");
               break;
            case 5:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_Q_MGR");
               break;
            case 6:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_CHANNEL");
               break;
            case 7:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_AUTH_INFO");
               break;
            case 8:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_TOPIC");
               break;
            case 9:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_COMM_INFO");
               break;
            case 10:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_CF_STRUC");
               break;
            case 11:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_LISTENER");
               break;
            case 12:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_SERVICE");
               break;
            case 1001:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_ALL");
               break;
            case 1002:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_ALIAS_Q");
               break;
            case 1003:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_MODEL_Q");
               break;
            case 1004:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_LOCAL_Q");
               break;
            case 1005:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_REMOTE_Q");
               break;
            case 1007:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_SENDER_CHANNEL");
               break;
            case 1008:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_SERVER_CHANNEL");
               break;
            case 1009:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_REQUESTER_CHANNEL");
               break;
            case 1010:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_RECEIVER_CHANNEL");
               break;
            case 1011:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_CURRENT_CHANNEL");
               break;
            case 1012:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_SAVED_CHANNEL");
               break;
            case 1013:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_SVRCONN_CHANNEL");
               break;
            case 1014:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_CLNTCONN_CHANNEL");
               break;
            case 1015:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_SHORT_CHANNEL");
               break;
            case 1016:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_CHLAUTH");
               break;
            case 1017:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_REMOTE_Q_MGR_NAME");
               break;
            case 1019:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_PROT_POLICY");
               break;
            case 1020:
                TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_TT_CHANNEL");
                break;
            case 1021:
                TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_AMQP_CHANNEL");
                break;
            case 1022:
                TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType MQOT_AUTH_REC");
                break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "ObjectType UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "OriginalLength" :
			if (field == -1)
				TrcParser.writeOutputFile(trcCtl, padStr + "OriginalLength MQOL_UNDEFINED");
			break;
		case "PD.Context" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Context MQPD_NO_CONTEXT");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Context MQPD_USER_CONTEXT");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Context UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "Persistence" :
			switch (field) {
            case -1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Persistence MQPER_PERSISTENCE_AS_PARENT");
               break;
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Persistence MQPER_NOT_PERSISTENT");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Persistence MQPER_PERSISTENT");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "Persistence MQPER_PERSISTENCE_AS_Q_DEF");
               TrcParser.writeOutputFile(trcCtl, padStr + "Persistence MQPER_PERSISTENCE_AS_TOPIC_DEF");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Persistence UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "Priority" :
			switch (field) {
            case -1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Priority MQPRI_PRIORITY_AS_Q_DEF");
               TrcParser.writeOutputFile(trcCtl, padStr + "Priority MQPRI_PRIORITY_AS_TOPIC_DEF");
               break;
            case -2:
               TrcParser.writeOutputFile(trcCtl, padStr + "Priority MQPRI_PRIORITY_AS_PARENT");
               break;
            case -3:
               TrcParser.writeOutputFile(trcCtl, padStr + "Priority MQPRI_PRIORITY_AS_PUBLISHED");
               break;
            }
			break;
		case "PubPriority" :
            switch (field) {
            case -1:
            	TrcParser.writeOutputFile(trcCtl, padStr + "PubPriority MQPRI_PRIORITY_AS_Q_DEF");
            	TrcParser.writeOutputFile(trcCtl, padStr + "PubPriority MQPRI_PRIORITY_AS_TOPIC_DEF");
               break;
            case -2:
            	TrcParser.writeOutputFile(trcCtl, padStr + "PubPriority MQPRI_PRIORITY_AS_PARENT");
               break;
            case -3:
            	TrcParser.writeOutputFile(trcCtl, padStr + "PubPriority MQPRI_PRIORITY_AS_PUBLISHED");
               break;
            }
			break;
		case "PutApplType" :
			switch (field) {
			case -1:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_UNKNOWN");
                break;
             case 0:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_NO_CONTEXT");
                break;
             case 1:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_CICS");
                break;
             case 2:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_MVS");
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_OS390");
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_ZOS");
                break;
             case 3:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_IMS");
                break;
             case 4:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_OS2");
                break;
             case 5:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_DOS");
                break;
             case 6:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_AIX");
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_UNIX");
                break;
             case 7:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_QMGR");
                break;
             case 8:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_OS400");
                break;
             case 9:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_WINDOWS");
                break;
             case 10:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_CICS_VSE");
                break;
             case 11:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_WINDOWS_NT");
                break;
             case 12:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_VMS");
                break;
             case 13:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_GUARDIAN");
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_NSK");
                break;
             case 14:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_VOS");
                break;
             case 15:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_OPEN_TP1");
                break;
             case 18:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_VM");
                break;
             case 19:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_IMS_BRIDGE");
                break;
             case 20:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_XCF");
                break;
             case 21:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_CICS_BRIDGE");
                break;
             case 22:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_NOTES_AGENT");
                break;
             case 23:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_TPF");
                break;
             case 25:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_USER");
                break;
             case 26:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_BROKER");
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_QMGR_PUBLISH");
                break;
             case 28:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_JAVA");
                break;
             case 29:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_DQM");
                break;
             case 30:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_CHANNEL_INITIATOR");
                break;
             case 31:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_WLM");
                break;
             case 32:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_BATCH");
                break;
             case 33:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_RRS_BATCH");
                break;
             case 34:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_SIB");
                break;
             case 35:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_SYSTEM_EXTENSION");
                break;
             case 36:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_MCAST_PUBLISH");
                break;
             case 37:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType MQAT_AMQP");
                break;
             default:
                TrcParser.writeOutputFile(trcCtl, padStr + "PutApplType UNKNOWN_CONSTANT");
                break;
			}
			break;
		case "PutMsgRecFields" :
            if (field == 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "PutMsgRecFields MQPMRF_NONE");
             if ((field & 0x1) == 0x1)
                TrcParser.writeOutputFile(trcCtl, padStr + "PutMsgRecFields MQPMRF_MSG_ID");
             if ((field & 0x2) == 0x2)
                TrcParser.writeOutputFile(trcCtl, padStr + "PutMsgRecFields MQPMRF_CORREL_ID");
             if ((field & 0x4) == 0x4)
                TrcParser.writeOutputFile(trcCtl, padStr + "PutMsgRecFields MQPMRF_GROUP_ID");
             if ((field & 0x8) == 0x8)
                TrcParser.writeOutputFile(trcCtl, padStr + "PutMsgRecFields MQPMRF_FEEDBACK");
             if ((field & 0x10) == 0x10)
                TrcParser.writeOutputFile(trcCtl, padStr + "PutMsgRecFields MQPMRF_ACCOUNTING_TOKEN");
             if ((field & 0xFFFFFFE0) != 0)
                TrcParser.writeOutputFile(trcCtl, padStr + "PutMsgRecFields UNKNOWN_CONSTANT in range 0xFFFFFFE0");
			break;
		case "ReconnectDelay" :
            switch (field) {
            case -1:
               TrcParser.writeOutputFile(trcCtl, padStr + "ReconnectDelay MQRD_NO_RECONNECT");
               break;
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "ReconnectDelay MQRD_NO_DELAY");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "ReconnectDelay UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "Report" :
			if (field == 0)
				TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_NONE");
			else 
			{
				switch (field & 0x07000000) {
				case 16777216:
					TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_EXCEPTION");
					break;
				case 50331648:
					TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_EXCEPTION_WITH_DATA");
					break;
				case 117440512:
					TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_EXCEPTION_WITH_FULL_DATA");
					break;
				}

				switch (field & 0x00E00000) {
				case 2097152:
					TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_EXPIRATION");
		            break;
				case 6291456:
					TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_EXPIRATION_WITH_DATA");
		            break;
				case 14680064:
					TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_EXPIRATION_WITH_FULL_DATA");
		            break;
				}

		        switch (field & 0x00000700) {
		        case 256:
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_COA");
		            break;
		        case 768:
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_COA_WITH_DATA");
		            break;
		        case 1792:
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_COA_WITH_FULL_DATA");
		            break;
		        }

		        switch (field & 0x00003800) {
		        case 2048:
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_COD");
		            break;
		        case 6144:
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_COD_WITH_DATA");
		            break;
		        case 14336:
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_COD_WITH_FULL_DATA");
		            break;
		        }

		        if ((field & 0x1) == 0x1)
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_PAN");
		        if ((field & 0x2) == 0x2)
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_NAN");
		        if ((field & 0x4) == 0x4)
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_ACTIVITY");
		        if ((field & 0x80) == 0x80)
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_PASS_MSG_ID");
		        if ((field & 0x40) == 0x40)
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_PASS_CORREL_ID");
		        if ((field & 0x08000000) == 0x08000000)
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_DISCARD_MSG");
		        if ((field & 0x00004000) == 0x00004000)
		        	TrcParser.writeOutputFile(trcCtl, padStr + "Report MQRO_PASS_DISCARD_AND_EXPIRY");
			}
			break;
		case "ResolvedType" :
			switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "ResolvedType MQOT_NONE");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "ResolvedType MQOT_Q");
               break;
            case 8:
               TrcParser.writeOutputFile(trcCtl, padStr + "ResolvedType MQOT_TOPIC");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "ResolvedType UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "Segmentation" :
            switch (str1) {
            case " ":
               TrcParser.writeOutputFile(trcCtl, padStr + "Segmentation MQSEG_INHIBITED");
               break;
            case "A":
               TrcParser.writeOutputFile(trcCtl, padStr + "Segmentation MQSEG_ALLOWED");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Segmentation UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "SegmentStatus" :
            switch (str1) {
            case " ":
               TrcParser.writeOutputFile(trcCtl, padStr + "SegmentStatus MQSS_NOT_A_SEGMENT");
               break;
            case "S":
               TrcParser.writeOutputFile(trcCtl, padStr + "SegmentStatus MQSS_SEGMENT");
               break;
            case "L":
               TrcParser.writeOutputFile(trcCtl, padStr + "SegmentStatus MQSS_LAST_SEGMENT");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "SegmentStatus UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "Signal1" :
            switch (field) {
            case 0:
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "Signal1 MQEC_MSG_ARRIVED");
               break;
            case 3:
               TrcParser.writeOutputFile(trcCtl, padStr + "Signal1 MQEC_WAIT_INTERVAL_EXPIRED");
               break;
            case 4:
               TrcParser.writeOutputFile(trcCtl, padStr + "Signal1 MQEC_WAIT_CANCELED");
               break;
            case 5:
               TrcParser.writeOutputFile(trcCtl, padStr + "Signal1 MQEC_Q_MGR_QUIESCING");
               break;
            case 6:
               TrcParser.writeOutputFile(trcCtl, padStr + "Signal1 MQEC_CONNECTION_QUIESCING");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Signal1 UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "StatType" :
            switch (field) {
            case 0:
            	TrcParser.writeOutputFile(trcCtl, padStr + "StatType MQSTAT_TYPE_ASYNC_ERROR");
               break;
            case 1:
            	TrcParser.writeOutputFile(trcCtl, padStr + "StatType MQSTAT_TYPE_RECONNECTION");
               break;
            case 2:
            	TrcParser.writeOutputFile(trcCtl, padStr + "StatType MQSTAT_TYPE_RECONNECTION_ERROR");
               break;
            default:
            	TrcParser.writeOutputFile(trcCtl, padStr + "StatType UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "State" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "State MQCS_NONE");
               break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "State MQCS_SUSPENDED_TEMPORARY");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "State MQCS_SUSPENDED_USER_ACTION");
               break;
            case 3:
               TrcParser.writeOutputFile(trcCtl, padStr + "State MQCS_SUSPENDED");
               break;
            case 4:
               TrcParser.writeOutputFile(trcCtl, padStr + "State MQCS_STOPPED");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "State UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "SubExpiry" :
            if (field == -1)
            	TrcParser.writeOutputFile(trcCtl, padStr + "Expiry MQEI_UNLIMITTED");
			break;
		case "Support" :
            switch (field) {
            case -1048576:
                TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_REJECT_UNSUP_MASK");
                break;
            case 0:
                TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_NONE");
                break;
            case 1:
               TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_SUPPORT_OPTIONAL");
               break;
            case 24:
                TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_CURRENT_LENGTH");
                TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_LENGTH_1");
                break;
            case 1023:
                TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_ACCEPT_UNSUP_MASK");
                break;
            case 1024:
                TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_SUPPORT_REQUIRED_IF_LOCAL");
                break;
            case 1047552:
                TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_ACCEPT_UNSUP_IF_XMIT_MASK");
                break;
            case 1048576:
               TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD_SUPPORT_REQUIRED");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Support MQPD UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "Type" :
            switch (field) {
            case 0:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_AS_SET");
               break;
            case 2:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_NULL");
               break;
            case 4:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_BOOLEAN");
               break;
            case 8:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_BYTE_STRING");
               break;
            case 16:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_INT8");
               break;
            case 32:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_INT16");
               break;
            case 64:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_INT32");
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_LONG");
               break;
            case 128:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_INT64");
               break;
            case 256:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_FLOAT32");
               break;
            case 512:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_FLOAT64");
               break;
            case 1024:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type MQTYPE_STRING");
               break;
            default:
               TrcParser.writeOutputFile(trcCtl, padStr + "Type UNKNOWN_CONSTANT");
               break;
            }
			break;
		case "WaitInterval" :
			if (field == -1)
				TrcParser.writeOutputFile(trcCtl, padStr + "WaitInterval MQWI_UNLIMITED");
			break;
		default :
			break;
		}
	}
	
}
